local API = GetLibResearchAPIWrapper()
local Config = GetLibResearchConfig()

RB2.CharacterSettings = {}

function RB2.CharacterSettings:new(charName, comparisonCharacters)
	assert(charName)
	
	local obj = {}
	setmetatable(obj, self)
	self.__index = self
	
	obj.CharacterName = charName
	obj.ComparisonCharacters = comparisonCharacters and comparisonCharacters or {}
	obj.Top = -1
	obj.Left = -1
	obj.PanelResearchAnchor = nil
	obj.PanelKnowledgeAnchor = nil
	obj.Locked = false
	obj.ControllerShown = true
	obj.ControllerFadeTime = 5
	obj.ReminderWasShown = false
	
	obj:initTrackers()
	
	return obj;
end

function RB2.CharacterSettings:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self

	obj.Top = obj.Top or 0
	obj.Left = obj.Left or 0
	if (obj.Locked == nil) then
		obj.Locked = false
	end
	if (obj.ControllerShown == nil) then	
		obj.ControllerShown = true
	end
	obj.ControllerFadeTime = obj.ControllerFadeTime or 5
	if (obj.ReminderWasShown == nil) then
		obj.ReminderWasShown = false
	end
	
	obj:initTrackers()
end

function RB2.CharacterSettings:resetUIRelatedSettings()
	self.Top = -1
	self.Left = -1
	self.PanelResearchAnchor = nil
	self.PanelKnowledgeAnchor = nil
	self.Locked = false
	self.ControllerShown = true
	self.ControllerFadeTime = 5
	self.ReminderWasShown = false
end

function RB2.CharacterSettings:initTrackers()
	if (self.Trackers == nil) then
		self.Trackers = {}
	end
	for i = 1,#Config.TradeSkillTypes do
		local tst = Config.TradeSkillTypes[i].Type
		if (self.Trackers[tst] == nil) then
			self.Trackers[tst] = true
		end
	end	
end

function RB2.CharacterSettings:mergeComparisonCharacterList(comparisonCharacterList)
	RB2.Debug.debug("mergeComparisonCharacterList")
	local keyTable = {}
	
	if (self.ComparisonCharacters == nil) then
		self.ComparisonCharacters = {}
	end
	
	if (self.ComparisonCharacters ~= nil) then
		for i = 1, #self.ComparisonCharacters do
			RB2.Debug.debug("Adding existing character: " .. self.ComparisonCharacters[i])
			keyTable[self.ComparisonCharacters[i]] = 1
		end
	end
	
	local charactersToAdd = {}
	if (comparisonCharacterList ~= nil) then
		for i = 1, #comparisonCharacterList do
			local char = comparisonCharacterList[i]
			RB2.Debug.debug("Checking char: " .. char)
			if (not keyTable[char]) then
				RB2.Debug.debug("not found, adding")
				table.insert(charactersToAdd, char)
			end
		end
	end
	
	for i = 1, #charactersToAdd do
		table.insert(self.ComparisonCharacters, charactersToAdd[i])
	end	
	RB2.Debug.debug("end")
end

function RB2.CharacterSettings:getCharacterName()
	return self.CharacterName
end

function RB2.CharacterSettings:setCharacterName(charName)
	self.CharacterName = charName
end

function RB2.CharacterSettings:getComparisonCharacters()
	return self.ComparisonCharacters
end

function RB2.CharacterSettings:getComparisonCharacterNameAtIndex(index)
	if ((self.ComparisonCharacters == nil) or (self.ComparisonCharacters[index] == nil) or (self.ComparisonCharacters[index] == "")) then
		return nil
	else
		return self.ComparisonCharacters[index]
	end
end

function RB2.CharacterSettings:setComparisonCharacterNameAtIndex(index, charName)
	if (self.ComparisonCharacters == nil) then
		self.ComparisonCharacters = {}
	end
	
	if (self.CharacterName == charName) then
		return
	end

	if (index > #self.ComparisonCharacters) then
		table.insert(self.ComparisonCharacters, charName)
	else
		self.ComparisonCharacters[index] = charName
	end
	
	local tmp = {}
	
	for i = 1, #self.ComparisonCharacters do
		local char = self.ComparisonCharacters[i]
		if (tmp[char] == nil) then
			tmp[char] = 1
		else
			self.ComparisonCharacters[i] = nil
		end
	end
	
	local idx = 1
	while (idx <= #self.ComparisonCharacters) do
		if (self.ComparisonCharacters[idx] == nil) then
			table.remove(self.ComparisonCharacters, idx)
		else
			idx = idx + 1
		end
	end	
end

function RB2.CharacterSettings:getFirstComparisonCharacter()
	return self:getComparisonCharacterNameAtIndex(1)
end

function RB2.CharacterSettings:setFirstComparisonCharacter(charName)
	return self:setComparisonCharacterNameAtIndex(1, charName)
end

function RB2.CharacterSettings:getSecondComparisonCharacter()
	return self:getComparisonCharacterNameAtIndex(2)
end

function RB2.CharacterSettings:setSecondComparisonCharacter(charName)
	return self:setComparisonCharacterNameAtIndex(2, charName)
end

function RB2.CharacterSettings:getThirdComparisonCharacter()
	return self:getComparisonCharacterNameAtIndex(3)
end

function RB2.CharacterSettings:setThirdComparisonCharacter(charName)
	return self:setComparisonCharacterNameAtIndex(3, charName)
end

function RB2.CharacterSettings:getTracker1()
	return self.Trackers[Config.TradeSkillTypes[1].Type]
end

function RB2.CharacterSettings:setTracker1(tracker)
	self.Trackers[Config.TradeSkillTypes[1].Type] = tracker
end

function RB2.CharacterSettings:getTracker2()
	return self.Trackers[Config.TradeSkillTypes[2].Type]
end

function RB2.CharacterSettings:setTracker2(tracker)
	self.Trackers[Config.TradeSkillTypes[2].Type] = tracker
end

function RB2.CharacterSettings:getTracker3()
	return self.Trackers[Config.TradeSkillTypes[3].Type]
end

function RB2.CharacterSettings:setTracker3(tracker)
	self.Trackers[Config.TradeSkillTypes[3].Type] = tracker
end

function RB2.CharacterSettings:getTracker(idx)
	return self.Trackers[Config.TradeSkillTypes[idx].Type]
end

function RB2.CharacterSettings:storeMainPanelPositions(top, left)
	self.Top = top
	self.Left = left
	RB2.Debug.debug("Stored main, top: " .. top .. ", left: " .. left)
end

function RB2.CharacterSettings:storeResearchPanelAnchor(anchorData)
	self.PanelResearchAnchor = anchorData
end

function RB2.CharacterSettings:storeKnowledgePanelAnchor(anchorData)
	self.PanelKnowledgeAnchor = anchorData
end

function RB2.CharacterSettings:mirrorUISettingsFrom(mirrorCharsettings)
	self.Top = mirrorCharsettings.Top
	self.Left = mirrorCharsettings.Left
	self.PanelResearchAnchor = mirrorCharsettings.PanelResearchAnchor
	self.PanelKnowledgeAnchor = mirrorCharsettings.PanelKnowledgeAnchor
	self.Locked = mirrorCharsettings.Locked
	self.ControllerShown = mirrorCharsettings.ControllerShown
	self.ControllerFadeTime = mirrorCharsettings.ControllerFadeTime
	self.ReminderWasShown = mirrorCharsettings.ReminderWasShown
end
