local Utils = GetLibResearchUtils() 

function RB2.eventHandlerLootReceived(eventCode, ...)
	RB2.Debug.debug("eventHandlerLootReceived")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventoryAndDisplayNewItemLinks, ...)
end

function RB2.eventHandlerMailAttachmentTaken(eventCode, ...)
	RB2.Debug.debug("eventHandlerMailAttachmentTaken")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventoryAndDisplayNewItemLinks, ...)
end

function RB2.eventHandlerFullUpdate(eventCode, ...)
	RB2.Debug.debug("eventHandlerFullUpdate")
	if(not RB2.DisableInventoryRefresh) then
		RB2.DisableInventoryRefresh = true
		zo_callLater(RB2.performDelayedFullUpdate, 1000)
	end
end

function RB2.eventHandlerSingleSlotUpdate(eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason)
	if (updateReason ~= INVENTORY_UPDATE_REASON_DEFAULT) then
		return		
	end
	
	if(not RB2.DisableInventoryRefresh) then
		RB2.Debug.debug("eventHandlerSingleSlotUpdate")	
		--	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventoryAndDisplayNewItemLinks, ...)
		-- no throttle to support deconstruction + multiple items display correctly
		RB2.refreshInventoryAndDisplayNewItemLinks()
	end
end

function RB2.eventHandlerCraftCompleted(eventCode, ...)
	RB2.Debug.debug("eventHandlerItemDestroyed")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventory, ...)
end

function RB2.eventHandlerItemDestroyed(eventCode, ...)
	RB2.Debug.debug("eventHandlerItemDestroyed")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventory, ...)
end

function RB2.eventHandlerResearchStarted(eventCode, ...)
	RB2.Debug.debug("eventHandlerResearchStarted")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventoryAndResearchProgress, ...)
end

function RB2.eventHandlerResearchCompleted(eventCode, ...)
	RB2.Debug.debug("eventHandlerResearchCompleted")
	RB2.throttleEventHandler(eventCode, RB2.Buffers["RB2_" .. eventCode], RB2.refreshInventoryAndResearchProgress, ...)
end

function RB2.eventHandlerStyleLearned(eventCode, ...)
	RB2.Debug.debug("eventHandlerStyleLearned")
	-- no need to throttle
	RB2.refreshMotifKnowledge()
	RB2.UI.PKMotifGridBeforeShow()
end

function RB2.eventHandlerTradingHouseOpen(eventCode, ...)
	RB2.disableInventoryFunction()
end

function RB2.eventHandlerTradingHouseClose(eventCode, ...)
	RB2.enableInventoryFunction()
end

function RB2.eventHandlerTradingHouseAwaitingResponse(eventCode, ...)
	RB2.Debug.debug("Awaiting response")
	RB2.disableInventoryFunction()
end

function RB2.eventHandlerTradingHouseSearchResultsReceived(eventCode, _, numItems, ...)
	RB2.Debug.debug("Results received")
	if (numItems > 0) then
		RB2.enableInventoryFunction()
	else
		RB2.disableInventoryFunction()
	end
end

function RB2.eventHandlerCombatState(eventCode, inCombat)
	if (inCombat) then
		RB2.Debug.debug("Entering combat")
		RB2.DisableInventoryRefresh = true
		RB2.UI.HideControllerWithUI()
	else
		RB2.Debug.debug("Leaving combat")
		RB2.DisableInventoryRefresh = false
		RB2.UI.ShowControllerWithUI()
	end
end

function RB2.eventHandlerStableInteractStart(eventCode, ...)
	RB2.Debug.debug("eventHandlerStableInteractStart")
	RB2.DisableInventoryRefresh = true
end

function RB2.eventHandlerStableInteractEnd(eventCode, ...)
	RB2.Debug.debug("eventHandlerStableInteractEnd")
	RB2.DisableInventoryRefresh = false
end

-------------------------------------------

function RB2.refreshInventoryAndResearchProgress()
	RB2.refreshInventory()
	RB2.refreshResearchProgress()
end

function RB2.refreshInventory()
--	local shouldRefresh = RB2.DisableInventoryRefresh or RB2.LibRes.APIWrapper.IsPlayerDead() or RB2.LibRes.APIWrapper.IsPlayerInCombat()

	if (not RB2.DisableInventoryRefresh) then
		RB2.Debug.debug("Refreshing inventory...")
		return RB2.CDM:refreshCurrentCharacterInventoryData(RB2.BankContainer)
	else
		RB2.Debug.debug("Skipping inventory refresh...")
	end
end

function RB2.refreshResearchProgress()
	RB2.Debug.debug("Refresing research progress...")
	RB2.CDM:refreshCurrentCharacterInProgressResearchData()
	RB2.refreshMotifKnowledge()
	RB2.UI.RefreshResearchIcons()
end

function RB2.refreshMotifKnowledge()
	RB2.Debug.debug("Refresing motif knowledge...")
	RB2.CDM:refreshCurrentCharacterMotifData(RB2.MRDC)
end

function RB2.displayNewItemLinks(newItemLinks)
	RB2.Debug.debug("displayNewItemLinks")
	if (RB2.SettingsObject.EnableChatOutput == false) then
		return
	end
	if (newItemLinks == nil) then
		return
	end

	for i = 1,#newItemLinks do
		local itemLink = newItemLinks[i]
		local resKey, tst, rli, ti, valid = RB2.LibRes.Utils.getResearchKeyFromItemLink(itemLink)
		if (valid) then
			local rpi = RB2.CDM:getResearchProgressInfoOfCurrentCharacter(resKey)
			if (rpi.status == rpi.StatusType.UNKNOWN) then
				local rrd = RB2.RRDC:getResearchRefDataByKey(resKey)
				local prettyItemLink = RB2.LibRes.Utils.prettifyItemName(itemLink)
				RB2.chatOutput(prettyItemLink .. RB2.Colors.Teal .. " providing " .. RB2.Colors.White .. rrd.researchLineTraitDescription .. RB2.Colors.Teal .. " has been acquired!")
			end
		end
	end
end

function RB2.refreshInventoryAndDisplayNewItemLinks(...)
	RB2.Debug.debug("refreshInventoryAndDisplayNewItemLinks called")
	RB2.displayNewItemLinks(RB2.refreshInventory())
end

function RB2.initLoot(x, control, ...)
	RB2.addResearchInfoToControl(control, ...)	
	return false
end

function RB2.eventChatterBegin()
	if (RB2.LibRes.APIWrapper.IsPlayerBeingArrested()) then
		RB2.Debug.debug("Disabling inventory refresh")
		RB2.DisableInventoryRefresh = true
	end
end

function RB2.eventChatterEnd()
	if (RB2.DisableInventoryRefresh) then
		RB2.Debug.debug("Enabling inventory refresh")
		RB2.DisableInventoryRefresh = false
		RB2.refreshInventoryAndDisplayNewItemLinks()
	end
end

function RB2.performDelayedFullUpdate()
	RB2.Debug.debug("performDelayedFullUpdate")
	RB2.DisableInventoryRefresh = false
	RB2.refreshInventoryAndDisplayNewItemLinks()	
end

function RB2.displayMotifResearchInfoToControl(itemLink, researchIcon, comparisonCharacterName)
	if (researchIcon ~= nil) then

		local parts = RB2.LibRes.APIWrapper.ParseLink(itemLink)
		local name = parts[1]
		local itemID = parts[4]
		local styleItemIndex = -1

		if (RB2.Knowledge.MotifIDs["" .. itemID] ~= nil) then
			styleItemIndex = RB2.Knowledge.MotifIDs["" .. itemID][1]			
		end

		if (styleItemIndex == -1) then
			researchIcon:SetHidden(true)
			RB2.Debug.error("Book " .. name .. " (" .. itemID .. ") is not found!")
			return
		end

		local known = false
		local hide = false

		local mkc = RB2.CDM:getMKCfOCharacter(comparisonCharacterName)
		if (mkc == nil) then
			researchIcon:SetHidden(true)
			RB2.Debug.error("Couldn't find motif knowledge container for character: " .. comparisonCharacterName)
			return
		end
	
		-- Dwemer (chapters are not really supported by the API, only for the current character via IsItemLinkBookKnown)
		if (styleItemIndex == 15) then
			local dwemerIndex = RB2.Knowledge.MotifIDs["" .. itemID][2]
			known = mkc:isChapterKnown(dwemerIndex)			
		else
			known = mkc:isStyleItemIndexKnown(styleItemIndex)
		end

		if (hide) then
			researchIcon:SetHidden(true)
		else
			if (known) then
				researchIcon.data = { tooltipText = comparisonCharacterName .. " - Known" }
				researchIcon:SetHidden(false)
				researchIcon:SetColor(255, 255, 255, 1)
				researchIcon:SetAlpha(0.5)
			else
				researchIcon.data = { tooltipText = comparisonCharacterName .. " - Unknown" }
				researchIcon:SetHidden(false)
				researchIcon:SetColor(255, 0, 0, 1)
				researchIcon:SetAlpha(1)
			end
		end
	end
end

local function getStoreSpecificItemData(control)
	local controlName = control:GetName()
	
	for k,v in pairs(RB2.ListingFunctions) do
		local st,_ = string.find(controlName, k:GetName())
		if ((st ~= nil) and (st == 1)) then
			return v[2](v[1], control)
		end
	end
end

function RB2.getBackpackItemData(bagId, control)
	local parent = control:GetParent()
	-- override bagId, because some panels show multiple bags, like deconstruction
	local bid = parent["dataEntry"] ~= nil and
		parent["dataEntry"]["data"] ~= nil and
		parent["dataEntry"]["data"]["bagId"] ~= nil and
		parent["dataEntry"]["data"]["bagId"] or nil
	local slotIndex = parent["dataEntry"] ~= nil and
		parent["dataEntry"]["data"] ~= nil and
		parent["dataEntry"]["data"]["slotIndex"] ~= nil and
		parent["dataEntry"]["data"]["slotIndex"] or nil
	local itemLink = nil
	local totalCount = 1
	local junk = false
	local tradingHouse = false
	local isValid = false
	local isIGV = false

	if (slotIndex ~= nil) then
		isValid = true
		itemLink = RB2.LibRes.APIWrapper.GetItemLinkBrackets(bid, slotIndex)
		totalCount = RB2.LibRes.APIWrapper.GetItemTotalCount(bid, slotIndex)
		junk = RB2.LibRes.APIWrapper.IsItemJunk(bid, slotIndex)

		if ((InventoryGridViewSettings ~= nil) and (bagId > 0)) then
			isIGV = InventoryGridViewSettings:IsGrid(bagId)
		end
	end

	return isValid, bagId, slotIndex, itemLink, totalCount, junk, tradingHouse, isIGV	
end

function RB2.getTradingHouseItemData(bagId, control)
	local parent = control:GetParent()
	local slotIndex = parent["dataEntry"] ~= nil and
		parent["dataEntry"]["data"] ~= nil and
		parent["dataEntry"]["data"]["slotIndex"] ~= nil and
		parent["dataEntry"]["data"]["slotIndex"] or nil
	local itemLink = nil
	local totalCount = 1
	local junk = false
	local tradingHouse = false
	local isValid = false

	if (slotIndex ~= nil) then
		isValid = true
		-- GetTradingHouseSearchResultItemLinkWorkaround is needed because of the line below
		itemLink = RB2.LibRes.APIWrapper.GetTradingHouseSearchResultItemLinkBrackets(slotIndex)
		-- GetTradingHouseSearchResultItemLinkWorkaround is needed because of the line above
		tradingHouse = true
	end

	return isValid, bagId, slotIndex, itemLink, totalCount, junk, tradingHouse, false	
end

function RB2.getLootItemData(bagId, control)
	local parent = control:GetParent()
	local lootId = parent["dataEntry"] ~= nil and
		parent["dataEntry"]["data"] ~= nil and
		parent["dataEntry"]["data"]["lootId"] ~= nil and
		parent["dataEntry"]["data"]["lootId"] or nil
	local itemLink = nil
	local totalCount = 1
	local junk = false
	local tradingHouse = false
	local isValid = false

	if (lootId ~= nil) then
		isValid = true
		itemLink = RB2.LibRes.APIWrapper.GetLootItemLink(lootId)
		tradingHouse = false
	end

	return isValid, bagId, lootId, itemLink, totalCount, junk, tradingHouse, false	
end

local IGVAnchorPositions = {
	{ TOPLEFT, TOPLEFT },
	{ BOTTOMRIGHT, BOTTOMRIGHT },
	{ TOPRIGHT, TOPRIGHT },
	{ BOTTOMLEFT, BOTTOMLEFT },
}

function RB2.addResearchInfoToControl(control, ...)

	if (not	RB2.InventoryFunctionEnabled) then
		return false
	end

	if (control == nil) then
		RB2.Debug.error("Inventory hook received no control!")
		return false
	end
	
	local parent = control:GetParent()
	local parentName = parent:GetName()
	local researchIcon = nil

	local isValid, bagId, slotIndex, itemLink, totalCount, junk, tradingHouse, isIGV = getStoreSpecificItemData(control)
	
	if (not isValid) then
		for i = 1,4 do
			local researchControlName = parentName .. "RB2ResearchIcon" .. i
			researchIcon = _G[researchControlName]

			if (researchIcon ~= nil) then
				researchIcon:SetHidden(true)
			end
		end
		RB2.Debug.debug("Invalid page, not displaying")
		return false
	end

	local anchorControlName = parentName .. "Name"
	if (isIGV) then
		anchorControlName = parentName .. "Highlight"
	end
	local anchorControl = _G[anchorControlName]

	if (anchorControl ~= nil) then
		local charactersToCompareTo = { RB2.LibRes.APIWrapper.GetCharacterName() }
		local additionalCharacters = RB2.CDM:getComparisonCharacters()
		for i = 1,#additionalCharacters do
			table.insert(charactersToCompareTo, additionalCharacters[i])
		end

		for i = 1,#RB2.Icons do

			local comparisonCharacterName = charactersToCompareTo[i]
			local researchControlName = parentName .. "RB2ResearchIcon" .. i
			researchIcon = _G[researchControlName]

			if (researchIcon == nil) then
				RB2.Debug.debug(researchControlName .. ": Icon doesn't exist, creating...")
				researchIcon = WINDOW_MANAGER:CreateControl(researchControlName, parent, CT_TEXTURE)
				researchIcon.data = { tooltipText = "" }
				researchIcon:SetTexture(RB2.Icons[i]) --"/esoui/art/icons/servicemappins/servicepin_smithy.dds")
				researchIcon:SetInheritAlpha(true)
				researchIcon:SetInheritScale(true)
				researchIcon:SetMouseEnabled(true)
				researchIcon:SetHandler("OnMouseEnter", ZO_Options_OnMouseEnter)
				researchIcon:SetHandler("OnMouseExit", ZO_Options_OnMouseExit)
			end
			
			-- repositioning
			researchIcon:ClearAnchors()
			if (isIGV) then
				researchIcon:SetDimensions(16, 16)
				local paddingX = 0
				local paddingY = 0
				local igvAP = IGVAnchorPositions[i]
				researchIcon:SetAnchor(igvAP[1], anchorControl, igvAP[2], paddingX, paddingY)
			else
				researchIcon:SetDimensions(24, 24)
				local paddingX = -24 + (i-1) * 16
				local paddingY = 0
				if (tradingHouse) then
					paddingX = -72 + (i-1) * 16
					paddingY = 8
				end
				researchIcon:SetAnchor(LEFT, anchorControl, RIGHT, paddingX, paddingY)
			end

			local found = false
			local hasMultiple = false
			local temporary = false
			local rpi = nil

			if (comparisonCharacterName ~= nil) then
				local resKey, tst, rli, ti, valid = RB2.LibRes.Utils.getResearchKeyFromItemLink(itemLink)
				RB2.Debug.debug("Itemlink: " .. itemLink)
				if (valid) then
					RB2.Debug.debug("Valid item found")
					RB2.Debug.debug("Reskey: " .. resKey)
					rpi = RB2.CDM:getResearchProgressInfoOfCharacter(comparisonCharacterName, resKey)
					if (rpi ~= nil) then
						found = true
						local researchKeyNumbers = RB2.CDM:haveMultipleItemsForTheSameTrait(resKey)
						researchKeyNumbers = RB2.BankContainer:hasMultipleItemsForTheSameTrait(resKey, researchKeyNumbers)
						if ((researchKeyNumbers[resKey] ~= nil) and (researchKeyNumbers[resKey] > 1)) then
							hasMultiple = true
						end
					end
				end
			end

			if (found) then
				if (researchIcon ~= nil) then
					if ((rpi.status == rpi.StatusType.KNOWN) or (rpi.status == rpi.StatusType.COMPLETED)) then
						researchIcon.data = { tooltipText = comparisonCharacterName .. " - Known" }
						researchIcon:SetColor(255, 255, 255, 1)
						researchIcon:SetAlpha(0.5)
					elseif (rpi.status == rpi.StatusType.INPROGRESS) then
						researchIcon.data = { tooltipText = comparisonCharacterName .. " - In progress" }
						researchIcon:SetColor(0, 0, 255, 1)
						researchIcon:SetAlpha(1)
					elseif (rpi.status == rpi.StatusType.UNKNOWN) then
						if (hasMultiple) then
							researchIcon.data = { tooltipText = comparisonCharacterName .. " - Unknown - Duplicate" }
							researchIcon:SetColor(255, 255, 0, 1)
						else
							researchIcon.data = { tooltipText = comparisonCharacterName .. " - Unknown" }
							researchIcon:SetColor(255, 0, 0, 1)
						end
						researchIcon:SetAlpha(1)
					end

					researchIcon:SetHidden(false)
				else
					RB2.Debug.error("Wanted to display research icon but the control doesn't exist!")
				end
			else

				local itemType = RB2.LibRes.APIWrapper.GetItemLinkItemType(itemLink)
				if ((itemType == ITEMTYPE_RACIAL_STYLE_MOTIF) and (comparisonCharacterName ~= nil)) then
					RB2.displayMotifResearchInfoToControl(itemLink, researchIcon, comparisonCharacterName)
				else
					if (researchIcon ~= nil) then
						researchIcon:SetHidden(true)
					end
				end
			end
		end
	else
		if (researchIcon ~= nil) then
			researchIcon:SetHidden(true)
		end
	end

	return false
end

--[[
Tab switch handling tip by sirinsidiator
]]--
function RB2.handleTabSwitchInTradingHouse(self, tabData)
	local mode = tabData.descriptor
	RB2.Debug.debug("Mode: " .. mode)
	if(RB2.TradingDisabledTabs[mode] ~= nil) then
		RB2.disableInventoryFunction()
	else
		RB2.enableInventoryFunction()
	end
end
