local LogLevel = 1
local LogLevels = {
	DEBUG = 1,
	INFO = 2,
	ERROR = 3,
	NOLOG = 4,
}
local LogLevelNames = { "Debug", "Info", "Error", "No logging" }
local MessageBuffer = {}
local MaxMessageNumber = 300
local LogColors = { "|c00FF00", "|c00FFFF", "|cFFFF00" }

local Debug = {
	MessageBuffer = MessageBuffer,
	LogLevels = LogLevels,
	LogLevelNames = LogLevelNames,
}

function Debug.getLogLevel()
	return LogLevel
end

function Debug.setLogLevel(level)
	local cnt = 0
	for k,v in pairs(LogLevels) do
		cnt = cnt + 1
	end
	
	Debug.info("Setting loglevel to " .. level .. " (max: " .. cnt .. ")")	
	if ((level >= 1) and (level <= cnt)) then
		Debug.info("Loglevel set.")
		LogLevel = level
	end
end

function Debug.setLogLevelByName(level)
	local levelNumber = 0
	for i = 1, #LogLevelNames do
		if (level == LogLevelNames[i]) then
			levelNumber = i
			break
		end
	end
	
	if (levelNumber ~= 0) then
		Debug.setLogLevel(levelNumber)
	end
end

function Debug.getMaxMessageNumber()
	return MaxMessageNumber
end

function Debug.setMaxMessageNumber(number)
	if (number < 1) then
		error("Number is less than 1!")
	end
	MaxMessageNumber = number
end

function Debug.getMessages()
	return MessageBuffer
end

function Debug.log(data, level, category)
	if (level > LogLevels.NOLOG) then
		level = LogLevels.NOLOG
	end
	if (level < LogLevel) then
		return
	end

	local msg = "|cFFFFFFLRD "
	if (category ~= nil) then
		msg = msg .. "[" .. category .. "]"
	end
	msg = msg .. ": " .. LogColors[level]

	local typ = type(data)
	if (typ == "table") then
		msg = msg .. "!"
		for k,v in pairs(data) do
			msg = msg .. k .. "=" .. v .. "!"
		end
	elseif (typ == "userdata") then
		if (data.GetName) then
			msg = msg .. "Userdata: " .. data:GetName()
		else
			msg = msg .. "Unknown Userdata received!"
		end
	else
		msg = msg .. data
	end

	d(msg)

	table.insert(MessageBuffer, msg)
	while ((#MessageBuffer > MaxMessageNumber) and (#MessageBuffer > 0)) do
		table.remove(MessageBuffer, 1)
	end
end

function Debug.debug(data, category)
	Debug.log(data, LogLevels.DEBUG, category)
end

function Debug.info(data, category)
	Debug.log(data, LogLevels.INFO, category)
end

function Debug.error(data, category)
	Debug.log(data, LogLevels.ERROR, category)
end

function GetLibResearchDebug()
	return Debug
end
