local API = GetLibResearchAPIWrapper()
local Debug = GetLibResearchDebug()
local Utils = GetLibResearchUtils()

local InventoryContainer = {}

function InventoryContainer:new(charName, bagType)
	assert(charName)
	assert(bagType)
	local obj = {}
	setmetatable(obj, self)
	self.__index = self

	obj.BagType = bagType
	obj.ItemLinks = {}
	obj.ResearchKeys = {}
	obj.CharacterName = charName

	return obj;
end

function InventoryContainer:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self

	if (obj.BagType == nil) then
		obj.BagType = BAG_BACKPACK
	end
end

function InventoryContainer:refresh()
	local oldItemLinks = self.ItemLinks
	self.ItemLinks = {}
	local newItemLinks = {}	
	self.ResearchKeys = {}

	local bagSize = API.GetBagSize(self.BagType)
	for i = 0, bagSize-1 do
		local itemLink = API.GetItemLinkBrackets(self.BagType, i)
		if ((itemLink ~= nil) and (itemLink ~= "")) then
			local resKey, tst, rli, ti, valid = Utils.getResearchKeyFromItemLink(itemLink)
			if (valid) then
				self.ItemLinks[itemLink] = resKey
				if (self.ResearchKeys[resKey] == nil) then
					self.ResearchKeys[resKey] = {}
				end
				table.insert(self.ResearchKeys[resKey], itemLink)
				
				if (oldItemLinks[itemLink] == nil) then
					table.insert(newItemLinks, itemLink)
				end
			end
		end
	end
	
	return newItemLinks
end

function InventoryContainer:hasMultipleItemsForTheSameTrait(researchKey, existingResults)
	if (self.ResearchKeys == nil) then
		self.ResearchKeys = {}
	end
	
	local result = {}
	if (existingResults ~= nil) then
		for k,v in pairs(existingResults) do result[k] = v end
	end
	
	if (self.ResearchKeys[researchKey] ~= nil) then
		for i = 1,#self.ResearchKeys[researchKey] do
			local il = self.ResearchKeys[researchKey][i]
			if (result[researchKey] == nil) then
				Debug.debug("Setting " .. researchKey .. " to 1 ")
				result[researchKey] = 1
			else
				Debug.debug("Incrementing " .. researchKey .. " to 1 ")
				result[researchKey] = result[researchKey] + 1
			end
		end
	end
	
	return result
end

function InventoryContainer:hasItemForResearchKey(researchKey)
	if (self.ResearchKeys == nil) then
		self.ResearchKeys = {}
	end

	local result = false

	if (self.ResearchKeys[researchKey] ~= nil) then
		result = (#self.ResearchKeys[researchKey] > 0)
	end	
	
	return result
end

function GetLibResearchInventoryContainer()
	return InventoryContainer
end
