local API = GetLibResearchAPIWrapper()
local debug = GetLibResearchDebug()
local Utils = GetLibResearchUtils()

local MotifKnowledgeContainer = {}
local MotifDwemerChapters = {
	Chapters = {
		ITEM_STYLE_CHAPTER_AXES, 		-- 10
		ITEM_STYLE_CHAPTER_MACES,		-- 9
		ITEM_STYLE_CHAPTER_SWORDS,		-- 8
		ITEM_STYLE_CHAPTER_DAGGERS,		-- 11		
		ITEM_STYLE_CHAPTER_BOWS,		-- 14
		ITEM_STYLE_CHAPTER_STAVES,		-- 12
		ITEM_STYLE_CHAPTER_SHIELDS,		-- 13
		ITEM_STYLE_CHAPTER_CHESTS,		-- 5
		ITEM_STYLE_CHAPTER_BOOTS,		-- 3
		ITEM_STYLE_CHAPTER_GLOVES,		-- 2
		ITEM_STYLE_CHAPTER_HELMETS,		-- 1
		ITEM_STYLE_CHAPTER_LEGS,		-- 4
		ITEM_STYLE_CHAPTER_SHOULDERS,	-- 7
		ITEM_STYLE_CHAPTER_BELTS,		-- 6
	},
	ChapterIcons = {
		[ITEM_STYLE_CHAPTER_AXES] = "/esoui/art/icons/gear_breton_1haxe_a.dds",
		[ITEM_STYLE_CHAPTER_MACES] = "/esoui/art/icons/gear_breton_1hhammer_a.dds",
		[ITEM_STYLE_CHAPTER_SWORDS] = "/esoui/art/icons/gear_breton_1hsword_a.dds",		
		[ITEM_STYLE_CHAPTER_DAGGERS] = "/esoui/art/icons/gear_breton_dagger_a.dds",
		[ITEM_STYLE_CHAPTER_BOWS] = "/esoui/art/icons/gear_breton_bow_a.dds",
		[ITEM_STYLE_CHAPTER_STAVES] = "/esoui/art/icons/gear_breton_staff_a.dds",
		[ITEM_STYLE_CHAPTER_SHIELDS] = "/esoui/art/icons/gear_breton_shield_a.dds",
		[ITEM_STYLE_CHAPTER_CHESTS] = "/esoui/art/icons/gear_breton_heavy_chest_a.dds",
		[ITEM_STYLE_CHAPTER_BOOTS] = "/esoui/art/icons/gear_breton_heavy_feet_a.dds",
		[ITEM_STYLE_CHAPTER_GLOVES] = "/esoui/art/icons/gear_breton_heavy_hands_a.dds",
		[ITEM_STYLE_CHAPTER_HELMETS] = "/esoui/art/icons/gear_breton_heavy_head_b.dds",
		[ITEM_STYLE_CHAPTER_LEGS] = "/esoui/art/icons/gear_breton_heavy_legs_a.dds",
		[ITEM_STYLE_CHAPTER_SHOULDERS] = "/esoui/art/icons/gear_breton_heavy_shoulders_a.dds",
		[ITEM_STYLE_CHAPTER_BELTS] = "/esoui/art/icons/gear_breton_heavy_waist_a.dds",
	},
	ChapterNames = {
		[ITEM_STYLE_CHAPTER_AXES] = "Axes",
		[ITEM_STYLE_CHAPTER_MACES] = "Maces",
		[ITEM_STYLE_CHAPTER_SWORDS] = "Swords",		
		[ITEM_STYLE_CHAPTER_DAGGERS] = "Daggers",
		[ITEM_STYLE_CHAPTER_BOWS] = "Bows",
		[ITEM_STYLE_CHAPTER_STAVES] = "Staves",
		[ITEM_STYLE_CHAPTER_SHIELDS] = "Shields",
		[ITEM_STYLE_CHAPTER_CHESTS] = "Chests",
		[ITEM_STYLE_CHAPTER_BOOTS] = "Boots",
		[ITEM_STYLE_CHAPTER_GLOVES] = "Gloves",
		[ITEM_STYLE_CHAPTER_HELMETS] = "Helmets",
		[ITEM_STYLE_CHAPTER_LEGS] = "Legs",
		[ITEM_STYLE_CHAPTER_SHOULDERS] = "Shoulders",
		[ITEM_STYLE_CHAPTER_BELTS] = "Belts",
	},
	BitMasks = {
		[ITEM_STYLE_CHAPTER_HELMETS]	= 128, --1,
		[ITEM_STYLE_CHAPTER_GLOVES]		= 64, --2,
		[ITEM_STYLE_CHAPTER_BOOTS]		= 4, --3,
		[ITEM_STYLE_CHAPTER_LEGS]		= 256, --4,
		[ITEM_STYLE_CHAPTER_CHESTS]		= 16, --5,
		[ITEM_STYLE_CHAPTER_BELTS]		= 2, --6,
		[ITEM_STYLE_CHAPTER_SHOULDERS]	= 2048, --7,
		[ITEM_STYLE_CHAPTER_SWORDS]		= 8192, --8,
		[ITEM_STYLE_CHAPTER_MACES]		= 512, --9,
		[ITEM_STYLE_CHAPTER_AXES]		= 1, --10,
		[ITEM_STYLE_CHAPTER_DAGGERS]	= 32, --11,
		[ITEM_STYLE_CHAPTER_STAVES]		= 4096, --12,
		[ITEM_STYLE_CHAPTER_SHIELDS]	= 1024, --13,
		[ITEM_STYLE_CHAPTER_BOWS]		= 8, --14,
	}
}
local DwemerAchievementID = 1144

function MotifKnowledgeContainer:new(charName)
	assert(charName)
	local obj = {}
	setmetatable(obj, self)
	self.__index = self

	obj.Motifs = {}
	obj.DwemerChapters = { }
	obj.CharacterName = charName

	return obj;
end

function MotifKnowledgeContainer:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self
	
	if (obj.DwemerChapters == nil) then
		obj.DwemerChapters = { }
	end
	if (obj.DwemerChapters == 0) then
		obj.DwemerChapters = { }
	end
end

function MotifKnowledgeContainer:refresh(motifRefDataContainer)
	if (self.CharacterName == API.GetCharacterName()) then
		self.Motifs = {}
		for k,v in pairs(motifRefDataContainer.MotifRefDataTable) do
			local known = API.IsSmithingStyleKnown(v.StyleItemIndex)
			self.Motifs[v.StyleItemIndex] = known
		end
		local bits = self:getDwemerChapterBits()
		for i = 1, #MotifDwemerChapters.Chapters do
			local idx = MotifDwemerChapters.Chapters[i]
			self.DwemerChapters[idx] = self:checkChapterKnown(bits, idx)
		end
	end
end

function MotifKnowledgeContainer:getDwemerChapterBits()
	local link = API.GetAchievementLink(DwemerAchievementID)
	local parts = API.ParseLink(link)
	local chapterBits = tonumber(parts[5])
	return chapterBits
end

function MotifKnowledgeContainer:getChapterIndexes()
	return MotifDwemerChapters.Chapters
end

function MotifKnowledgeContainer:getChapterIcon(chapterIndex)
	return MotifDwemerChapters.ChapterIcons[chapterIndex]
end

function MotifKnowledgeContainer:getChapterName(chapterIndex)
	return MotifDwemerChapters.ChapterNames[chapterIndex]
end

function MotifKnowledgeContainer:isChapterKnown(chapterIndex)
	return self.DwemerChapters[chapterIndex]
end

function MotifKnowledgeContainer:checkChapterKnown(bits, chapterIndex)
	local bitMask = MotifDwemerChapters.BitMasks[chapterIndex]
	local num = Utils.bitwiseAnd(bits, bitMask, #MotifDwemerChapters.Chapters)
	return num == bitMask
end

function MotifKnowledgeContainer:isStyleItemIndexKnown(styleItemIndex)
	return self.Motifs[styleItemIndex]
end

function MotifKnowledgeContainer:getMotifs()
	return self.Motifs
end

function GetLibResearchMotifKnowledgeContainer()
	return MotifKnowledgeContainer
end
