local API = GetLibResearchAPIWrapper()
local debug = GetLibResearchDebug()
local config = GetLibResearchConfig()
local resProgInfo = GetLibResearchResearchProgressInfo()

local ResearchProgressContainer = {
	InProgressTypes = {
		INPROGRESS = 1,
		COMPLETED = 2
	}
}

function ResearchProgressContainer:new(charName)
	assert(charName)
	local obj = {}
	setmetatable(obj, self)
	self.__index = self

	obj.Researches = {}
	obj.InProgressResearchKeys = { {}, {} }
	obj.CharacterName = charName
	
	obj:refreshMaxResearches()

	return obj;
end

function ResearchProgressContainer:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self

	for k,v in pairs(obj.Researches) do
		resProgInfo:reSetMetaTable(v)
	end
	
--	if (obj.CharacterName == "Leyanie Lightweaver") then
--		local key = "6_5_1"
--		local rpi = obj.Researches[key]
----		if (obj.InProgressResearchKeys[1][key] == nil) then
--			rpi.refreshTime = API.GetTimeStamp()
--			rpi.remaining = 15
--			rpi.status = rpi.StatusType.INPROGRESS
--			obj.InProgressResearchKeys[1][key] = 1		
----		end
--	end
	
	obj:refreshMaxResearches()
	
end

function ResearchProgressContainer:add(researchProgressInfo)
	local key = researchProgressInfo:getKey()
	if (self.Researches[key] == nil) then
		self.Researches[key] = researchProgressInfo
		self:refresh(researchProgressInfo)
	else
		local old = self.Researches[key]
		old:cloneFrom(researchProgressInfo)
	end
end

function ResearchProgressContainer:addAsInProgress(researchProgressInfo, type)
	--	debug.debug(researchProgressInfo:toString() .. ", type: " .. type)
	local key = researchProgressInfo:getKey()
	if (self.InProgressResearchKeys[type][key] == nil) then
		self.InProgressResearchKeys[type][key] = 1
		--		debug.debug(self.InProgressResearchKeys[type])
	end
end

function ResearchProgressContainer:removeInProgress(researchProgressInfo, type)
	local key = researchProgressInfo:getKey()
	self.InProgressResearchKeys[type][key] = nil
end

function ResearchProgressContainer:refreshMaxResearches()
	if (self.CharacterName == API.GetCharacterName()) then
		self.MaxResearchesTable = {}
	
		for i = 1, #config.TradeSkillTypes do
			local t = config.TradeSkillTypes[i]
			self.MaxResearchesTable[t.Type] = API.GetMaxSimultaneousSmithingResearch(t.Type)
		end
	end
end

function ResearchProgressContainer:refreshAll()
	self.InProgressResearchKeys = { {}, {} }
	for key, rpi in pairs(self.Researches) do
		self:refresh(rpi)
	end
	self:refreshMaxResearches()
end

function ResearchProgressContainer:refreshInProgress()
	local iprk = self.InProgressResearchKeys[ResearchProgressContainer.InProgressTypes.INPROGRESS]
	self.InProgressResearchKeys[ResearchProgressContainer.InProgressTypes.INPROGRESS] = {}
	for k,_ in pairs(iprk) do
		self:refreshByKey(k)
	end
end

function ResearchProgressContainer:refreshByKey(researchKey)
	local rpi = self.Researches[researchKey]
	if (rpi ~= nil) then
		self:refresh(rpi)
	end
end

function ResearchProgressContainer:refresh(researchProgressInfo)
	if (self.CharacterName == API.GetCharacterName()) then
		self:refreshByFunc(researchProgressInfo, researchProgressInfo.refreshOwn)
	else
		self:refreshByFunc(researchProgressInfo, researchProgressInfo.refreshOther)
	end
end

function ResearchProgressContainer:refreshByFunc(researchProgressInfo, refreshFunc)
	self:removeInProgress(researchProgressInfo, ResearchProgressContainer.InProgressTypes.INPROGRESS)
	self:removeInProgress(researchProgressInfo, ResearchProgressContainer.InProgressTypes.COMPLETED)

	refreshFunc(researchProgressInfo)

	if (researchProgressInfo.status == researchProgressInfo.StatusType.INPROGRESS) then
		self:addAsInProgress(researchProgressInfo, ResearchProgressContainer.InProgressTypes.INPROGRESS)
	elseif (researchProgressInfo.status == researchProgressInfo.StatusType.COMPLETED) then
		self:addAsInProgress(researchProgressInfo, ResearchProgressContainer.InProgressTypes.COMPLETED)
	end
end

function ResearchProgressContainer:initFromResearchRefDataContainer(rrdContainer)
	assert(rrdContainer)
	for i = 1, #rrdContainer.RefDataTable do
		local rrd = rrdContainer.RefDataTable[i]
		local rpi = resProgInfo:new(rrd.tradeSkillType, rrd.researchLineIndex, rrd.traitIndex, self.CharacterName)
		self:add(rpi)
	end
end

function ResearchProgressContainer:getEarliestCompletionSeconds()
	local earliest = -1
	for key,_ in pairs(self.InProgressResearchKeys[ResearchProgressContainer.InProgressTypes.INPROGRESS]) do
		local rpi = self.Researches[key]
		local remaining = rpi:getCompletionTime()
		if (earliest == -1) then
			earliest = remaining
		elseif (earliest > remaining) then
			earliest = remaining
		end
	end

	return earliest
end

function ResearchProgressContainer:listInProgressResearches(tradeSkillType, sort)
	local tmpTable = {}

	if (sort == nil) then
		sort = true
	end

	for k,_ in pairs(self.InProgressResearchKeys[ResearchProgressContainer.InProgressTypes.INPROGRESS]) do
		local res = self.Researches[k]
		if ((tradeSkillType ~= nil) and (res.tradeSkillType == tradeSkillType)) then
			table.insert(tmpTable, res)
		elseif (tradeSkillType == nil) then 
			table.insert(tmpTable, res)
		end
	end

	if (sort) then
		if (self.CharacterName == API.GetCharacterName()) then
			table.sort(tmpTable, self.rpiSortByCompletionTime)
		else
			table.sort(tmpTable, self.rpiSortByProjectedCompletionTime)
		end
	end

	return tmpTable
end

function ResearchProgressContainer:sortInProgressResearches(researches)
	local result = researches
	table.sort(result, self.rpiSortBoth)
	return result
end

function ResearchProgressContainer.rpiSortBoth(first, second)
	local firstTime = first:getCompletionTime()
--	if (first.characterName == API.GetCharacterName()) then
--		firstTime = first:getRemainingTime()
--	else
--		firstTime = first:getProjectedCompletionTime()
--	end
	
	local secondTime = second:getCompletionTime()
--	if (second.characterName == API.GetCharacterName()) then
--		secondTime = second:getRemainingTime()
--	else
--		secondTime = second:getProjectedCompletionTime()
--	end
	
--	RB2.Debug.error(first.characterName .. ": " .. firstTime .. ", " .. second.characterName .. ": " .. secondTime)
	
	return firstTime < secondTime
end

function ResearchProgressContainer.rpiSortByCompletionTime(first, second)
	return first:getRemainingTime() < second:getRemainingTime()
end

function ResearchProgressContainer.rpiSortByProjectedCompletionTime(first, second)
	return first:getProjectedCompletionTime() < second:getProjectedCompletionTime()
end

function ResearchProgressContainer:listCompletedResearches()
	local tmpTable = {}

	for k,_ in pairs(self.InProgressResearchKeys[ResearchProgressContainer.InProgressTypes.COMPLETED]) do
		local res = self.Researches[k]
		table.insert(tmpTable, res)
	end

	return tmpTable
end

function ResearchProgressContainer:getMaxResearches(tradeSkillType)
	return self.MaxResearchesTable[tradeSkillType]
end

function ResearchProgressContainer:getByResearchKey(researchKey)
	return self.Researches[researchKey]
end

function ResearchProgressContainer:getCurrentResearches(tradeSkillType)
	local result = {}
	local ipr = self:listInProgressResearches()
	
	for i = 1,#ipr do
		local rpi = ipr[i]
		if (rpi.tradeSkillType == tradeSkillType) then
			table.insert(result, rpi)
		end
	end
	
	return result
end

function ResearchProgressContainer:countKnownTraitsForResKey(tradeSkillType, researchLineIndex)
	 local count = 0
	 for k,v in pairs(self.Researches) do
	 	if ((v.tradeSkillType == tradeSkillType) and (v.researchLineIndex == researchLineIndex)
	 		and ((v.status == resProgInfo.StatusType.KNOWN) or (v.status == resProgInfo.StatusType.COMPLETED))) then
	 		count = count + 1
	 	end
	 end 
	 return count
end

function GetLibResearchResearchProgressContainer()
	return ResearchProgressContainer
end
