local API = GetLibResearchAPIWrapper()
local staleSecondsLimit = 600 -- 10 mins

local ResearchProgressInfo = {
	StatusType = {
		UNKNOWN = 1,
		INPROGRESS = 2,
		KNOWN = 3,
		COMPLETED = 4,
	},
	StatusNames = {
		[1] = "Unknown",
		[2] = "In progress",
		[3] = "Known",
		[4] = "Completed",
	}
}

function ResearchProgressInfo:new(tradeSkillType, researchLineIndex, traitIndex, characterName)
	local obj = {}
	setmetatable(obj, self)
	self.__index = self
	obj.tradeSkillType = tradeSkillType or CRAFTING_TYPE_INVALID
	obj.researchLineIndex = researchLineIndex or 0
	obj.traitIndex = traitIndex or 0
	obj.characterName = characterName

	obj:refreshOwn()

	return obj;
end

function ResearchProgressInfo:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self
end

function ResearchProgressInfo:refreshOwn()
	local rlti = { API.GetSmithingResearchLineTraitInfo(self.tradeSkillType, self.researchLineIndex, self.traitIndex) }

	self.known = rlti[3]

	self.status = self.known and ResearchProgressInfo.StatusType.KNOWN or ResearchProgressInfo.StatusType.UNKNOWN

	local _, remaining = API.GetSmithingResearchLineTraitTimes(self.tradeSkillType, self.researchLineIndex, self.traitIndex)
	local oldRemaining = self.remaining
	-- Dirty hack to overcome the "deferred-logout-bug" bug
	-- When you log out with your character and the deferred logout dialog appears, between the counter reaching 0
	-- and the actual logout, GetSmithingResearchLineTraitTimes starts to return weird values. I only saw the example
	-- with roughly 46000 remaining, it starts counting down, when the logout timer reaches zero, it goes up to 90000. WTH?
	if (oldRemaining ~= nil) then
		if ((remaining ~= nil) and (oldRemaining > remaining)) then
			self.refreshTime = API.GetTimeStamp()
			self.remaining = remaining
		end	
	else 
		self.refreshTime = API.GetTimeStamp()
		self.remaining = remaining
	end

	if (remaining ~= nil) then
		self.status = ResearchProgressInfo.StatusType.INPROGRESS
	end
end

function ResearchProgressInfo:refreshOther()
	if (self.status == ResearchProgressInfo.StatusType.INPROGRESS) then
		if (self:getProjectedCompletionTime() < API.GetTimeStamp()) then
			self.status = ResearchProgressInfo.StatusType.COMPLETED
		end
	end
end

function ResearchProgressInfo.createKey(tradeSkillType, researchLineIndex, traitIndex)
	return tradeSkillType .. "_" .. researchLineIndex .. "_" .. traitIndex
end

function ResearchProgressInfo:getKey()
	return ResearchProgressInfo.createKey(self.tradeSkillType, self.researchLineIndex, self.traitIndex)
end

function ResearchProgressInfo:getSecondsSinceLastRefresh()
	return API.GetTimeStamp() - self.refreshTime
end

function ResearchProgressInfo:isStale()
	return self:getSecondsSinceLastRefresh() >= staleSecondsLimit
end

function ResearchProgressInfo:getProjectedCompletionTime()
	if (self.remaining ~= nil) then
		return self.remaining + self.refreshTime
	else
		return 0
	end
end

function ResearchProgressInfo:getCompletionTime()
	if (self.characterName == API.GetCharacterName()) then
		return self:getRemainingTime()
	else
		return self:getRemainingTime() - (API.GetTimeStamp()-self.refreshTime)
	end
end

function ResearchProgressInfo:getRemainingTime()
	if (self.remaining ~= nil) then
		return self.remaining
	else
		return 0
	end
end

--function ResearchProgressInfo:isInProgress(skipTimeCalculation)
--	local result = false
--	if (skipTimeCalculation) then
--		result = self.status == ResearchProgressInfo.StatusType.INPROGRESS
--	else
--		if (self.status == ResearchProgressInfo.StatusType.INPROGRESS) then
--			if (self:getProjectedCompletionTime() > API.GetTimeStamp()) then
--				result = true
--			end
--		end
--	end
--
--	return result
--end

function ResearchProgressInfo:equals(otherResearchProgressInfo)
	local result = true
	if (otherResearchProgressInfo == nil) then
		return false
	end

	for key, value in pairs(self) do
		if (value ~= otherResearchProgressInfo[key]) then
			result = false
			break
		end
	end

	return result
end

function ResearchProgressInfo:toString()

	local str = "[ResearchProgressInfo] TST: %d, RLI: %d, TI: %d, Char: %s, known: %s, status: %s, remaining: %d"

	return string.format(str,
		self.tradeSkillType,
		self.researchLineIndex,
		self.traitIndex,
		self.characterName,
		(self.known and "true" or "false"),
		ResearchProgressInfo.StatusNames[self.status],
		(self.remaining or -1))
end

function ResearchProgressInfo:cloneFrom(rpi)
	self.tradeSkillType = rpi.tradeSkillType
	self.researchLineIndex = rpi.researchLineIndex
	self.traitIndex = rpi.traitIndex
	self.characterName = rpi.characterName
	self.known = rpi.known
	self.status = rpi.status
	self.refreshTime = rpi.refreshTime
	self.remaining = rpi.remaining
end

function GetLibResearchResearchProgressInfo()
	return ResearchProgressInfo
end
