local API = GetLibResearchAPIWrapper()

local ResearchRefData = { }

function ResearchRefData:new(tradeSkillType, researchLineIndex, traitIndex)
	local obj = {}
	setmetatable(obj, self)
	self.__index = self
	obj.tradeSkillType = tradeSkillType or CRAFTING_TYPE_INVALID
	obj.researchLineIndex = researchLineIndex or 0
	obj.traitIndex = traitIndex or 0

	obj:enrich()

	return obj;
end

function ResearchRefData:enrich()
	local rli = { API.GetSmithingResearchLineInfo(self.tradeSkillType, self.researchLineIndex) }

	self.researchLineName = rli[1]
	self.icon = rli[2]

	local rlti = { API.GetSmithingResearchLineTraitInfo(self.tradeSkillType, self.researchLineIndex, self.traitIndex) }

	self.itemTraitType = rlti[1]
	self.researchLineTraitDescription = API.GetItemTraitTypeName(self.itemTraitType)
end

function ResearchRefData.createKey(tradeSkillType, researchLineIndex, traitIndex)
	return tradeSkillType .. "_" .. researchLineIndex .. "_" .. traitIndex
end

function ResearchRefData:getKey()
	return ResearchRefData.createKey(self.tradeSkillType, self.researchLineIndex, self.traitIndex)
end

function ResearchRefData:equals(otherResearchRefData)
	local result = true
	if (otherResearchRefData == nil) then
		return false
	end

	for key, value in pairs(self) do
		if (value ~= otherResearchRefData[key]) then
			result = false
			break
		end
	end

	return result
end

function ResearchRefData:toString()

	local str = "[ResearchRefData] TST: %d, RLI: %d, TI: %d, ITT: %d, RLN: %s, RLTD: %s"

	return string.format(str,
		self.tradeSkillType,
		self.researchLineIndex,
		self.traitIndex,
		self.itemTraitType,
		self.researchLineName,
		self.researchLineTraitDescription)
end

function GetLibResearchResearchRefData()
	return ResearchRefData
end
