local Utils = {}
local API = GetLibResearchAPIWrapper()
local Debug = GetLibResearchDebug()

-- { <tradeSkillType>, <rliPlus> }
local armorTypeTable = {
	[ARMORTYPE_HEAVY]   = { 1, 7, },
	[ARMORTYPE_LIGHT]   = { 2, 0, },
	[ARMORTYPE_MEDIUM]  = { 2, 7, },
	[ARMORTYPE_NONE]    = { -1, 0, }
}

-- { <tradeSkillType>, <researchLineIndex> }
local weaponTypeTable = {
	[WEAPONTYPE_AXE]                = { 1, 1, },
	[WEAPONTYPE_BOW]                = { 6, 1, },
	[WEAPONTYPE_DAGGER]             = { 1, 7, },
	[WEAPONTYPE_FIRE_STAFF]         = { 6, 2, },
	[WEAPONTYPE_FROST_STAFF]        = { 6, 3, },
	[WEAPONTYPE_HAMMER]             = { 1, 2, },
	[WEAPONTYPE_HEALING_STAFF]      = { 6, 5, },
	[WEAPONTYPE_LIGHTNING_STAFF]    = { 6, 4, },
	[WEAPONTYPE_NONE]               = { -1, -1, },
	[WEAPONTYPE_RUNE]               = { -1, -1, },
	[WEAPONTYPE_SHIELD]             = { 6, 6, },
	[WEAPONTYPE_SWORD]              = { 1, 3, },
	[WEAPONTYPE_TWO_HANDED_AXE]     = { 1, 4, },
	[WEAPONTYPE_TWO_HANDED_HAMMER]  = { 1, 5, },
	[WEAPONTYPE_TWO_HANDED_SWORD]   = { 1, 6, },
}

-- <researchLineIndex>
local equipTypeTable = {
	[EQUIP_TYPE_CHEST] = 1,
	[EQUIP_TYPE_COSTUME] = -1,
	[EQUIP_TYPE_FEET] = 2,
	[EQUIP_TYPE_HAND] = 3,
	[EQUIP_TYPE_HEAD] = 4,
	[EQUIP_TYPE_INVALID] = -1,
	[EQUIP_TYPE_LEGS] = 5,
	[EQUIP_TYPE_MAIN_HAND] = -1,
	[EQUIP_TYPE_NECK] = -1,
	[EQUIP_TYPE_OFF_HAND] = -1,
	[EQUIP_TYPE_ONE_HAND] = -1,
	[EQUIP_TYPE_RING] = -1,
	[EQUIP_TYPE_SHOULDERS] = 6,
	[EQUIP_TYPE_TWO_HAND] = -1,
	[EQUIP_TYPE_WAIST] = 7,
}

local traitTypeTable = {
	[ITEM_TRAIT_TYPE_ARMOR_STURDY] = 1,
	[ITEM_TRAIT_TYPE_ARMOR_IMPENETRABLE] = 2,
	[ITEM_TRAIT_TYPE_ARMOR_REINFORCED] = 3,
	[ITEM_TRAIT_TYPE_ARMOR_WELL_FITTED] = 4,
	[ITEM_TRAIT_TYPE_ARMOR_TRAINING] = 5,
	[ITEM_TRAIT_TYPE_ARMOR_INFUSED] = 6,
	[ITEM_TRAIT_TYPE_ARMOR_EXPLORATION] = 7,
	[ITEM_TRAIT_TYPE_ARMOR_DIVINES] = 8,
	[ITEM_TRAIT_TYPE_ARMOR_NIRNHONED] = 9,
	[ITEM_TRAIT_TYPE_WEAPON_POWERED] = 1,
	[ITEM_TRAIT_TYPE_WEAPON_CHARGED] = 2,
	[ITEM_TRAIT_TYPE_WEAPON_PRECISE] = 3,
	[ITEM_TRAIT_TYPE_WEAPON_INFUSED] = 4,
	[ITEM_TRAIT_TYPE_WEAPON_DEFENDING] = 5,
	[ITEM_TRAIT_TYPE_WEAPON_TRAINING] = 6,
	[ITEM_TRAIT_TYPE_WEAPON_SHARPENED] = 7,
	[ITEM_TRAIT_TYPE_WEAPON_WEIGHTED] = 8,
	[ITEM_TRAIT_TYPE_WEAPON_NIRNHONED] = 9,
}

function Utils.getResearchKeyFromItemLink(itemLink)
	local researchKey = nil
	local tradeSkillType = -1
	local researchLineIndex = -1
	local traitIndex = -1
	local valid = false

	if ((itemLink == nil) or (itemLink == "")) then
		return researchKey, tradeSkillType, researchLineIndex, traitIndex, valid
	end

	local itemType = API.GetItemLinkItemType(itemLink)
	local equipType = API.GetItemLinkEquipType(itemLink)

	if ( ((itemType ~= ITEMTYPE_ARMOR) and (itemType ~= ITEMTYPE_WEAPON)) or ((equipType == EQUIP_TYPE_NECK) or (equipType == EQUIP_TYPE_RING)) ) then
		Debug.debug("Item is not researchable: " .. itemLink)
		return researchKey, tradeSkillType, researchLineIndex, traitIndex, valid
	end

	RB2.Debug.debug("Item link type: " .. itemType)

	if (itemType == ITEMTYPE_ARMOR) then
		local armorType = API.GetItemLinkArmorType(itemLink)
		tradeSkillType = armorTypeTable[armorType][1]
		local rliPlus = armorTypeTable[armorType][2]

		researchLineIndex = equipTypeTable[equipType]
		if ((researchLineIndex ~= nil) or (researchLineIndex ~= -1)) then
			researchLineIndex = researchLineIndex + rliPlus
		else
			researchLineIndex = -1
		end


	elseif (itemType == ITEMTYPE_WEAPON) then
		local weaponType = API.GetItemLinkWeaponType(itemLink)
		tradeSkillType = weaponTypeTable[weaponType][1]
		researchLineIndex = weaponTypeTable[weaponType][2]
	end

	local traitInfo = API.GetItemLinkTraitInfo(itemLink)
	traitIndex = traitTypeTable[traitInfo]
	if ((traitIndex == nil) or (traitIndex == 0)) then
		traitIndex = -1
	end

	if ((tradeSkillType ~= -1) and (researchLineIndex ~= -1) and (traitIndex ~= -1)) then
		researchKey = tradeSkillType .. "_" .. researchLineIndex .. "_" .. traitIndex
		valid = true
	end

	return researchKey, tradeSkillType, researchLineIndex, traitIndex, valid
end

function Utils.boolToString(bool)
	return bool and "true" or "false"
end

function Utils.prettifyItemName(str)
	return string.gsub(str, "(.*)\^[a-z](.*)", "%1%2")
		--return zo_strformat
end

function Utils.formatTime(time, hideSeconds)
	local result = ""

	if (hideSeconds == nil) then
		hideSeconds = false
	end

	if (time ~= nil) then

		local seconds = 0
		local minutes = 0
		local hours = 0
		local days = 0

		days = math.floor(time / 86400)
		local remainder = time - days * 86400
		hours = math.floor(remainder / 3600)
		remainder = remainder - hours * 3600
		minutes = math.floor(remainder / 60)
		seconds = remainder - minutes * 60

		if (hideSeconds) then
			if (days > 0) then
				result = string.format("%dd, %02d:%02d", days, hours, minutes)
			else
				result = string.format("%02d:%02d", hours, minutes)
			end
		else
			if (days > 0) then
				result = string.format("%dd, %02d:%02d:%02d", days, hours, minutes, seconds)
			else
				result = string.format("%02d:%02d:%02d", hours, minutes, seconds)
			end
		end
	end

	return result
end

function Utils.tablePrint(tbl, indent)
	if not indent then indent = 0 end
	for k, v in pairs(tbl) do
		local formatting = string.rep("__", indent) .. k .. ": "
		if type(v) == "table" then
			d(formatting)
			Utils.tablePrint(v, indent+1)
		else
			d(formatting .. tostring(v))
		end
	end
end

function Utils.bitwiseAnd(a, b, numberOfBits)
	local tmp = 0
	for i = 1, numberOfBits do
		local div = 2^(i - 1)
		tmp = tmp + (((math.floor(a/div) % 2) * (math.floor(b/div) % 2)) * div)  
	end
	return tmp
end

function GetLibResearchUtils()
	return Utils
end
