local RRDC = GetLibResearchResearchRefDataContainer()
local MRDC = GetLibResearchMotifRefDataContainer()
local IC = GetLibResearchInventoryContainer()
local Utils = GetLibResearchUtils()

function RB2.initSavedVariables(self)
	RB2.SavedVars = ZO_SavedVars:NewAccountWide("ResearchBuddy2_SavedVariables", RB2.AddonConfig.SavedVariablesVersion, nil, RB2.DefaultSettings)
end

function RB2.initAddon(self)
	RB2.SavedZO_Inventory_SetupSlot =_G[RB2.InventoryFunctionIdentifier]

	if (RB2.SavedVars.CDM == nil) then
		RB2.SavedVars.CDM = RB2.LibRes.CharacterDataManager:new()
	else
		RB2.LibRes.CharacterDataManager:reSetMetaTable(RB2.SavedVars.CDM)
	end

	RB2.CDM = RB2.SavedVars.CDM

	RB2.RRDC = RRDC:new()
	RB2.MRDC = MRDC:new()
	RB2.BankContainer = IC:new("bank", BAG_BANK)


	-- reloading or initializing character settings
	local characterSettings = RB2.CDM:getCustomData()
	if (characterSettings == nil) then
		characterSettings = RB2.Settings:new()
	else
		RB2.Settings:reSetMetaTable(characterSettings)
	end
	characterSettings:refreshCurrentCharacterData(RB2.CDM:listOtherCharacters())
	RB2.SettingsObject = characterSettings
	RB2.CDM:setCustomData(RB2.SettingsObject)

	RB2.CDM:refreshCurrentCharacterData(RB2.RRDC, RB2.MRDC, RB2.BankContainer)
	RB2.CDM:setComparisonCharacters(RB2.SettingsObject:getCurrentCharacterData():getComparisonCharacters())

	RB2:ToggleSlashCommand("rb2", RB2.Commands.handler)

	RB2:RegisterEvent(EVENT_LOOT_RECEIVED, RB2.eventHandlerLootReceived)
	RB2:RegisterEvent(EVENT_MAIL_TAKE_ATTACHED_ITEM_SUCCESS, RB2.eventHandlerMailAttachmentTaken)
	
	RB2:RegisterEvent(EVENT_INVENTORY_SINGLE_SLOT_UPDATE, RB2.eventHandlerSingleSlotUpdate)
	RB2:RegisterEvent(EVENT_INVENTORY_FULL_UPDATE, RB2.eventHandlerFullUpdate)
	
	RB2:RegisterEvent(EVENT_CRAFT_COMPLETED, RB2.eventHandlerCraftCompleted)
	RB2:RegisterEvent(EVENT_INVENTORY_ITEM_DESTROYED, RB2.eventHandlerItemDestroyed)
	RB2:RegisterEvent(EVENT_SMITHING_TRAIT_RESEARCH_STARTED, RB2.eventHandlerResearchStarted)
	RB2:RegisterEvent(EVENT_SMITHING_TRAIT_RESEARCH_COMPLETED, RB2.eventHandlerResearchCompleted)
	RB2:RegisterEvent(EVENT_STYLE_LEARNED, RB2.eventHandlerStyleLearned)

	RB2:RegisterEvent(EVENT_STABLE_INTERACT_START, RB2.eventHandlerStableInteractStart)
	RB2:RegisterEvent(EVENT_STABLE_INTERACT_END, RB2.eventHandlerStableInteractEnd)
	
	RB2:RegisterEvent(EVENT_PLAYER_COMBAT_STATE, RB2.eventHandlerCombatState)

	RB2:RegisterEvent(EVENT_CHATTER_BEGIN, RB2.eventChatterBegin)
	RB2:RegisterEvent(EVENT_CHATTER_END, RB2.eventChatterEnd)
	
	ZO_PreHook(_G, RB2.InventoryFunctionIdentifier, RB2.addResearchInfoToControl)
	
	if (RB2.AddonConfig.GetTradingHouseSearchResultItemLinkWorkaround) then
		RB2:RegisterEvent(EVENT_OPEN_TRADING_HOUSE, RB2.eventHandlerTradingHouseOpen)
		RB2:RegisterEvent(EVENT_CLOSE_TRADING_HOUSE, RB2.eventHandlerTradingHouseClose)
		RB2:RegisterEvent(EVENT_TRADING_HOUSE_AWAITING_RESPONSE, RB2.eventHandlerTradingHouseAwaitingResponse)
		RB2:RegisterEvent(EVENT_TRADING_HOUSE_SEARCH_RESULTS_RECEIVED, RB2.eventHandlerTradingHouseSearchResultsReceived)

		ZO_PreHook(TRADING_HOUSE, RB2.TradingHouseTabSwitchIdentifier, RB2.handleTabSwitchInTradingHouse)
	end
	
	ZO_PreHook(ZO_GameMenu_InGame, "OnHide", RB2.UI.ShowControllerWithUI)
	ZO_PreHook(ZO_GameMenu_InGame, "OnShow", RB2.UI.HideControllerWithUI)
	ZO_PreHook(LOOT_WINDOW, "SetUpLootItem", RB2.initLoot)
	
	RB2.UI.InitUIControls()
end

function RB2.enableInventoryFunction()
	RB2.Debug.debug("Enabling inventory function")
	RB2.InventoryFunctionEnabled = true
end

function RB2.disableInventoryFunction()
	RB2.Debug.debug("Disabling inventory function")
	RB2.InventoryFunctionEnabled = false
end

RB2.ListingFunctions = {
	[ZO_PlayerInventoryBackpack]				= { 1, RB2.getBackpackItemData },
	[ZO_PlayerInventoryQuest]					= { 2, RB2.getBackpackItemData },
	[ZO_PlayerBankBackpack]						= { 3, RB2.getBackpackItemData },
	[ZO_GuildBankBackpack]						= { 4, RB2.getBackpackItemData },
	[ZO_StoreWindowList]						= { 5, RB2.getBackpackItemData },
	[ZO_BuyBackList]							= { 6, RB2.getBackpackItemData },
	[ZO_TradingHouse]							= { -1, RB2.getTradingHouseItemData },
	[ZO_Loot]									= { -2, RB2.getLootItemData },
	[ZO_SmithingTopLevelDeconstructionPanel]	= { -3, RB2.getBackpackItemData },
}

-- Initializing LibAddonMenu
RB2.LAM = LibStub("LibAddonMenu-2.0")

ZO_CreateStringId("SI_BINDING_NAME_RB2_TOGGLE_CONTROLLER", "Toggle controller panel")
ZO_CreateStringId("SI_BINDING_NAME_RB2_SHOW_CONTROLLER_TEMP", "Temporarily show controller panel")
ZO_CreateStringId("SI_BINDING_NAME_RB2_TOGGLE_ACTIVE_RESEARCHES_PANEL", "Toggle active researches panel")
ZO_CreateStringId("SI_BINDING_NAME_RB2_TOGGLE_RESEARCH_KNOWLEDGE_PANEL", "Toggle research knowledge panel")

-- Initializing Wykkyd's Framework

if (LWF3 ~= nil) then

	LWF3.REGISTER_FACTORY(RB2, false, true,
		function( self ) RB2.initSavedVariables( self ) end,
		function( self ) RB2.initAddon( self ) end,
		function( self ) RB2.SettingsHandler.initSettings( self ) end
	)

elseif (LWF4 ~= nil) then

	LWF4.REGISTER_FACTORY(RB2, false, true,
		function( self ) RB2.initSavedVariables( self ) end,
		function( self ) RB2.initAddon( self ) end,
		function( self ) RB2.SettingsHandler.initSettings( self ) end
	)

end

for k,v in pairs(RB2.Buffers) do
	RB2:BufferPause(k, 1)
end
