local API = GetLibResearchAPIWrapper()

RB2.Settings = {}

function RB2.Settings:new()
	local obj = {}
	setmetatable(obj, self)
	self.__index = self

	obj.LogLevel = RB2.Debug.LogLevels.ERROR
	obj.CharacterSettings = {}
	obj.EnableChatOutput = true
	obj.ShowOthercharactersOnKnowledgePanel = true
	obj.ShowOthercharactersOnResearchPanel = true
	RB2.Debug.setLogLevel(obj.LogLevel)

	return obj;
end

function RB2.Settings:reSetMetaTable(obj)
	if (obj == nil) then
		return
	end

	setmetatable(obj, self)
	self.__index = self
	
	if (obj.LogLevel == nil) then
		obj.LogLevel = RB2.Debug.LogLevels.ERROR
	end
	RB2.Debug.setLogLevel(obj.LogLevel)
	
	if (obj.EnableChatOutput == nil) then
		obj.EnableChatOutput = true
	end
	
	if (obj.ShowOthercharactersOnResearchPanel == nil) then
		obj.ShowOthercharactersOnResearchPanel = true
	end

	if (obj.ShowOthercharactersOnKnowledgePanel == nil) then
		obj.ShowOthercharactersOnKnowledgePanel = true
	end
	
	if(obj.CharacterSettings ~= nil) then
		for k,v in pairs(obj.CharacterSettings) do
			RB2.CharacterSettings:reSetMetaTable(v)
		end
	end
end

function RB2.Settings:refreshCurrentCharacterData(extraCharacterList)
	if (self.CharacterSettings == nil) then
		self.CharacterSettings = {}
	end
	
	local charName = API.GetCharacterName()
	local settings = nil
	
	if (self.CharacterSettings[charName] == nil) then
		settings = RB2.CharacterSettings:new(charName, extraCharacterList)
		self.CharacterSettings[charName] = settings
	else
		settings = self.CharacterSettings[charName]
--		settings:mergeComparisonCharacterList(extraCharacterList)
	end
end

function RB2.Settings:getCurrentCharacterData()
	local result = nil
	
	local charName = API.GetCharacterName()
	if ((self.CharacterSettings ~= nil) and (self.CharacterSettings[charName] ~= nil)) then
		result = self.CharacterSettings[charName]
	end
	
	return result
end

function RB2.Settings:getLogLevel()	
	return RB2.Debug.LogLevelNames[self.LogLevel]
end

function RB2.Settings:setLogLevel(logLevel)
	RB2.Debug.setLogLevelByName(logLevel)
	self.LogLevel = RB2.Debug.getLogLevel()
end

function RB2.Settings:getCharacterData(charName)
	return self.CharacterSettings[charName]
end

function RB2.Settings:storeMainPanelPositions(top, left)
	self:getCurrentCharacterData():storeMainPanelPositions(top, left)
end

function RB2.Settings:storeResearchPanelAnchor(anchorData)
	self:getCurrentCharacterData():storeResearchPanelAnchor(anchorData)
end

function RB2.Settings:storeKnowledgePanelAnchor(anchorData)
	self:getCurrentCharacterData():storeKnowledgePanelAnchor(anchorData)
end

function RB2.Settings:getLeft()
	return self:getCurrentCharacterData().Left
end

function RB2.Settings:getTop()
	return self:getCurrentCharacterData().Top
end

function RB2.Settings:getPanelResearchAnchor()
	return self:getCurrentCharacterData().PanelResearchAnchor
end

function RB2.Settings:getPanelKnowledgeAnchor()
	return self:getCurrentCharacterData().PanelKnowledgeAnchor
end

function RB2.Settings:getLocked()
	return self:getCurrentCharacterData().Locked
end

function RB2.Settings:getControllerShown()
	return self:getCurrentCharacterData().ControllerShown
end

function RB2.Settings:setControllerShown(controllerShown)
	self:getCurrentCharacterData().ControllerShown = controllerShown
--	if (RB2.UI.Controls.MainPanel ~= nil) then
--		RB2.UI.Controls.MainPanel:SetHidden(not controllerShown)
--	end
end

function RB2.Settings:getControllerFadeTime()
	return self:getCurrentCharacterData().ControllerFadeTime
end

function RB2.Settings:getReminderWasShown()
	return self:getCurrentCharacterData().ReminderWasShown
end

function RB2.Settings:resetReminderWasShown()
	self:getCurrentCharacterData().ReminderWasShown = false
end

function RB2.Settings:setReminderWasShown()
	self:getCurrentCharacterData().ReminderWasShown = true
end

function RB2.Settings:mirrorUISettingsFrom(characterName)
	RB2.Debug.debug("Mirroring from " .. characterName)
	local mirrorFrom = self:getCharacterData(characterName)
	if (mirrorFrom ~= nil) then
		for charName, settings in pairs(self.CharacterSettings) do
			if (settings ~= mirrorFrom) then
				RB2.Debug.debug("Mirroring to " .. charName)
				settings:mirrorUISettingsFrom(mirrorFrom)
			else
				RB2.Debug.debug("Skipping " .. charName)
			end
		end
	end
end

function RB2.Settings:resetUIRelatedSettings(characterName)
	self:getCharacterData(characterName):resetUIRelatedSettings()
end
