local API = GetLibResearchAPIWrapper()
local currentCharacter = API:GetCharacterName()
local optionPanel = nil

local lamControlSubMenuName = "RB2_LAM_SubMenu"
local lamControlFirstDropdownName = "RB2_LAM_First_DropDown"
local lamControlSecondDropdownName = "RB2_LAM_Second_DropDown"
local lamControlThirdDropdownName = "RB2_LAM_Third_DropDown"
local lamControlTracker1 = "RB2_LAM_Tracker1"
local lamControlTracker2 = "RB2_LAM_Tracker2"
local lamControlTracker3 = "RB2_LAM_Tracker3"

local optionNone = "None"

RB2.SettingsHandler = {}

function RB2.SettingsHandler.initSettings()

	local panelData = {
		type = "panel",
		name = RB2.AddonConfig.AddonName,
		displayName = RB2.AddonConfig.SettingsPanelName,
		author = RB2.AddonConfig.Author,
		version = RB2.AddonConfig.AddonVersion,
		registerForRefresh = true,
		registerForDefaults = true,
	}

	local optionsTable = {
		{
			type = "description",
			text = "Research Buddy 2 is a complete rewrite of Research Buddy. It trackes researches and researchable items on your characters.",
		},
		{
			type = "dropdown",
			name = "Log level",
			tooltip = "Sets the log level used by the addon",
			warning = "Settings this to Info or Debug will produce a lot of output in the chat window! Only recommended for debugging purposes.",
			choices = RB2.Debug.LogLevelNames,
			getFunc = RB2.SettingsHandler.getLogLevel,
			setFunc = RB2.SettingsHandler.setLogLevel,
		},
		{
			type = "checkbox",
			name = "Show comparison characters on RP",
			tooltip = "Sets whether to show comparison characters on the Research Panel",
			getFunc = RB2.SettingsHandler.getShowOthercharactersOnResearchPanel,
			setFunc = RB2.SettingsHandler.setShowOthercharactersOnResearchPanel,
		},
		{
			type = "checkbox",
			name = "Show comparison characters on KP",
			tooltip = "Sets whether to show comparison characters' knowledge on the Knowledge Panel",
			getFunc = RB2.SettingsHandler.getShowOthercharactersOnKnowledgePanel,
			setFunc = RB2.SettingsHandler.setShowOthercharactersOnKnowledgePanel,
		},
		{
			type = "checkbox",
			name = "Chat output",
			tooltip = "Sets whether to send notification to the chat window upon acquiring researchable gear",
			getFunc = RB2.SettingsHandler.getEnableChatOutput,
			setFunc = RB2.SettingsHandler.setEnableChatOutput,
		},
		{
			type = "dropdown",
			name = "Selected character",
			tooltip = "Changes the current selected character",
			choices = RB2.CDM:getCharacterNames(),
			getFunc = RB2.SettingsHandler.getCurrentCharacter,
			setFunc = RB2.SettingsHandler.setCurrentCharacter,
		},
		{
			type = "submenu",
			name = currentCharacter .. "'s - Settings",
			reference = lamControlSubMenuName,
			controls = {
				{
					type = "checkbox",
					name = "Lock panel",
					tooltip = "Sets whether the main panel is moveable or not",
					getFunc = RB2.SettingsHandler.getLocked,
					setFunc = RB2.SettingsHandler.setLocked,
				},
				{
					type = "checkbox",
					name = "Show panel",
					tooltip = "Sets whether the main panel is visible or not",
					getFunc = RB2.SettingsHandler.getControllerShown,
					setFunc = RB2.SettingsHandler.setControllerShown,
				},
				{
					type = "slider",
					name = "Main panel fade time",
					tooltip = "Sets how many seconds to temporarily show the main panel when it is not visible and one of the panels ",
					min = 1,
					max = 120,
					step = 5,					
					getFunc = RB2.SettingsHandler.getControllerFadeTime,
					setFunc = RB2.SettingsHandler.setControllerFadeTime,
				},
				{
					type = "button",
					name = "Mirror to other characters",
					tooltip = "Copy the above settings and panel position/attributes to every other character",
					func = RB2.SettingsHandler.mirror,
				},				
				{
					type = "button",
					name = "Reset panel position settings",
					tooltip = "Clears every customization related to positioning",
					warning = "Reloads UI!",
					func = RB2.SettingsHandler.reset,
				},				
				{
					type = "dropdown",
					name = "First comparison character",
					tooltip = "Sets the first comparison character who's research knowledge will be shown in inventory/guild store windows beside the current character",
					choices = RB2.SettingsHandler.getCharacterDropDownValues(),
					getFunc = RB2.SettingsHandler.getFirstComparisonCharacter,
					setFunc = RB2.SettingsHandler.setFirstComparisonCharacter,
					reference = lamControlFirstDropdownName,
				},
				{
					type = "dropdown",
					name = "Second comparison character",
					tooltip = "Sets the second comparison character who's research knowledge will be shown in inventory/guild store windows beside the current character",
					choices = RB2.SettingsHandler.getCharacterDropDownValues(),
					getFunc = RB2.SettingsHandler.getSecondComparisonCharacter,
					setFunc = RB2.SettingsHandler.setSecondComparisonCharacter,
					reference = lamControlSecondDropdownName,
				},
				{
					type = "dropdown",
					name = "Third comparison character",
					tooltip = "Sets the third comparison character who's research knowledge will be shown in inventory/guild store windows beside the current character",
					choices = RB2.SettingsHandler.getCharacterDropDownValues(),
					getFunc = RB2.SettingsHandler.getThirdComparisonCharacter,
					setFunc = RB2.SettingsHandler.setThirdComparisonCharacter,
					reference = lamControlThirdDropdownName,
				},
				{
					type = "checkbox",
					name = "Track Smithing researches",
					tooltip = "Sets whether to track Smithing active researches and show the smithing icon with colors on the main panel",
					getFunc = RB2.SettingsHandler.getTracker1,
					setFunc = RB2.SettingsHandler.setTracker1,
					reference = lamControlTracker1,
				},
				{
					type = "checkbox",
					name = "Track Clothing researches",
					tooltip = "Sets whether to track Clothing active researches and show the clothing icon with colors on the main panel",
					getFunc = RB2.SettingsHandler.getTracker2,
					setFunc = RB2.SettingsHandler.setTracker2,
					reference = lamControlTracker2,
				},
				{
					type = "checkbox",
					name = "Track Woodworking researches",
					tooltip = "Sets whether to track Woodworking active researches and show the woodworking icon with colors on the main panel",
					getFunc = RB2.SettingsHandler.getTracker3,
					setFunc = RB2.SettingsHandler.setTracker3,
					reference = lamControlTracker3,
				},
			},
		},
	}

	optionPanel = RB2.LAM:RegisterAddonPanel(RB2.AddonConfig.AddonID.."_LAM", panelData)
	RB2.LAM:RegisterOptionControls(RB2.AddonConfig.AddonID.."_LAM", optionsTable)

	RB2.SettingsHandler.setCurrentCharacter(currentCharacter)
end

function RB2.SettingsHandler.getLogLevel()
	return RB2.SettingsObject:getLogLevel()
end

function RB2.SettingsHandler.setLogLevel(logLevel)
	return RB2.SettingsObject:setLogLevel(logLevel)
end

function RB2.SettingsHandler.getCurrentCharacter()
	return currentCharacter
end

function RB2.SettingsHandler.setCurrentCharacter(characterName)
	currentCharacter = characterName
	local sm = _G[lamControlSubMenuName]
	if (sm ~= nil) then
		sm.data.name = currentCharacter .. "'s - Settings"
		_G[lamControlFirstDropdownName]:UpdateChoices(RB2.SettingsHandler.getCharacterDropDownValues())
		_G[lamControlSecondDropdownName]:UpdateChoices(RB2.SettingsHandler.getCharacterDropDownValues())
		_G[lamControlThirdDropdownName]:UpdateChoices(RB2.SettingsHandler.getCharacterDropDownValues())
	end
end

function RB2.SettingsHandler.getCharacterDropDownValues()
	local result = RB2.CDM:listOtherCharacters(currentCharacter)
	table.insert(result, optionNone)
	return result;
end

function RB2.SettingsHandler.getFirstComparisonCharacter()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getFirstComparisonCharacter() or optionNone
end

function RB2.SettingsHandler.setFirstComparisonCharacter(charName)
	if (charName == optionNone) then
		charName = nil
	end
	
	RB2.SettingsObject:getCharacterData(currentCharacter):setFirstComparisonCharacter(charName)
	RB2.SettingsHandler.updateCDMWithComparisonCharacters()
	RB2.UI.InitCharacterSelectControl()
	_G[lamControlFirstDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getFirstComparisonCharacter())
	_G[lamControlSecondDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getSecondComparisonCharacter())
	_G[lamControlThirdDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getThirdComparisonCharacter())
end

function RB2.SettingsHandler.getSecondComparisonCharacter()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getSecondComparisonCharacter() or optionNone
end

function RB2.SettingsHandler.setSecondComparisonCharacter(charName)
	if (charName == optionNone) then
		charName = nil
	end
	
	RB2.SettingsObject:getCharacterData(currentCharacter):setSecondComparisonCharacter(charName)
	RB2.SettingsHandler.updateCDMWithComparisonCharacters()
	RB2.UI.InitCharacterSelectControl()
	_G[lamControlFirstDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getFirstComparisonCharacter())
	_G[lamControlSecondDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getSecondComparisonCharacter())
	_G[lamControlThirdDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getThirdComparisonCharacter())
end

function RB2.SettingsHandler.getThirdComparisonCharacter()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getThirdComparisonCharacter() or optionNone
end

function RB2.SettingsHandler.setThirdComparisonCharacter(charName)
	if (charName == optionNone) then
		charName = nil
	end
	
	RB2.SettingsObject:getCharacterData(currentCharacter):setThirdComparisonCharacter(charName)
	RB2.SettingsHandler.updateCDMWithComparisonCharacters()
	RB2.UI.InitCharacterSelectControl()
	_G[lamControlFirstDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getFirstComparisonCharacter())
	_G[lamControlSecondDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getSecondComparisonCharacter())
	_G[lamControlThirdDropdownName].dropdown:SetSelectedItem(RB2.SettingsHandler.getThirdComparisonCharacter())
end

function RB2.SettingsHandler.updateCDMWithComparisonCharacters()
	RB2.CDM:setComparisonCharacters(RB2.SettingsObject:getCharacterData(currentCharacter):getComparisonCharacters(), currentCharacter)
end

function RB2.SettingsHandler.getTracker1()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getTracker1()
end

function RB2.SettingsHandler.setTracker1(value)
	RB2.SettingsObject:getCharacterData(currentCharacter):setTracker1(value)
	RB2.UI.RefreshResearchIcons()
end

function RB2.SettingsHandler.getTracker2()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getTracker2()
end

function RB2.SettingsHandler.setTracker2(value)
	RB2.SettingsObject:getCharacterData(currentCharacter):setTracker2(value)
	RB2.UI.RefreshResearchIcons()
end

function RB2.SettingsHandler.getTracker3()
	return RB2.SettingsObject:getCharacterData(currentCharacter):getTracker3()
end

function RB2.SettingsHandler.setTracker3(value)
	RB2.SettingsObject:getCharacterData(currentCharacter):setTracker3(value)
	RB2.UI.RefreshResearchIcons()
end

function RB2.SettingsHandler.getTracker(idx)
	return RB2.SettingsObject:getCharacterData(RB2.CDM.SelectedCharacter):getTracker(idx)
end

function RB2.SettingsHandler.getLocked()
	return RB2.SettingsObject:getCharacterData(currentCharacter).Locked
end

function RB2.SettingsHandler.setLocked(locked)
	RB2.SettingsObject:getCharacterData(currentCharacter).Locked = locked
	if (currentCharacter == API:GetCharacterName()) then
		RB2.UI.Controls.MainPanel:SetMovable(not locked)
		RB2.UI.Controls.PanelResearch:SetMovable(not locked)
		RB2.UI.Controls.PanelKnowledge:SetMovable(not locked)
	end
end

function RB2.SettingsHandler.getControllerShown()
	return RB2.SettingsObject:getCharacterData(currentCharacter).ControllerShown
end

function RB2.SettingsHandler.setControllerShown(shown)
	RB2.SettingsObject:getCharacterData(currentCharacter).ControllerShown = shown
	if (currentCharacter == API:GetCharacterName()) then
		RB2.SettingsObject:setControllerShown(shown)
	end
end

function RB2.SettingsHandler.getControllerFadeTime()
	return RB2.SettingsObject:getCharacterData(currentCharacter).ControllerFadeTime
end

function RB2.SettingsHandler.setControllerFadeTime(fadeTime)
	RB2.SettingsObject:getCharacterData(currentCharacter).ControllerFadeTime = fadeTime
end

function RB2.SettingsHandler.getEnableChatOutput()
	return RB2.SettingsObject.EnableChatOutput
end

function RB2.SettingsHandler.setEnableChatOutput(value)
	RB2.SettingsObject.EnableChatOutput = value
end

function RB2.SettingsHandler.getShowOthercharactersOnResearchPanel()
	return RB2.SettingsObject.ShowOthercharactersOnResearchPanel
end

function RB2.SettingsHandler.setShowOthercharactersOnResearchPanel(value)
	RB2.SettingsObject.ShowOthercharactersOnResearchPanel = value
	RB2.UI.RefreshResearchIcons()	
end

function RB2.SettingsHandler.getShowOthercharactersOnKnowledgePanel()
	return RB2.SettingsObject.ShowOthercharactersOnKnowledgePanel
end

function RB2.SettingsHandler.setShowOthercharactersOnKnowledgePanel(value)
	RB2.SettingsObject.ShowOthercharactersOnKnowledgePanel = value
end


function RB2.SettingsHandler.mirror()
	RB2.SettingsObject:mirrorUISettingsFrom(currentCharacter)
end

function RB2.SettingsHandler.reset()
	RB2.SettingsObject:resetUIRelatedSettings(currentCharacter)
	if (currentCharacter == API:GetCharacterName()) then
	    ReloadUI("ingame")
	end	
end
