local API = GetLibResearchAPIWrapper()
local Utils = GetLibResearchUtils()
local Config = GetLibResearchConfig()
local MoKnC = GetLibResearchMotifKnowledgeContainer()

RB2.UI = {}
RB2.UI.Controls = {}
RB2.UI.Controls.MotifData = {}
RB2.UI.Controls.MotifDwemerData = {}
RB2.UI.Controls.KnowledgeSummaries = {}
RB2.UI.InitializationPhase = 0
RB2.UI.PanelResearchOpened = false
RB2.UI.PanelKnowledgeOpened = false
RB2.UI.PanelKnowledgeRefreshStarted = false


function RB2.UI.CharacterSelectInitialize(control)
	ZO_ComboBox:New(control)
	control.dropdown = ZO_ComboBox_ObjectFromContainer(control)
	RB2.UI.Controls.CharacterSelectDropdown = control
	control.dropdown:SetSortsItems(false)
end

function RB2.UI.InitSmithingIcon(control)
	RB2.UI.Controls.IconSmithing = control
end

function RB2.UI.InitClothinIcon(control)
	RB2.UI.Controls.IconClothing = control
end

function RB2.UI.InitWoodworkingIcon(control)
	RB2.UI.Controls.IconWoodworking = control
end

function RB2.UI.InitResearchButton(control)
	RB2.UI.Controls.ButtonResearch = control
end

function RB2.UI.InitKnowledgeButton(control)
	RB2.UI.Controls.ButtonKnowledge = control
end

function RB2.UI.InitMainPanel(control)
	RB2.UI.Controls.MainPanel = control
	RB2.UI.ControllerTempShown = false
	
	RB2.UI.InitializationPhase = RB2.UI.InitializationPhase + 1
end

function RB2.UI.InitCharacterSelectControl()
	local control = RB2.UI.Controls.CharacterSelectDropdown

	control.dropdown:ClearItems()

	local selectedItem = nil
	local characters = RB2.CDM:getCharacterNames()
	for i = 1, #characters do
		local entry = control.dropdown:CreateItemEntry(characters[i], RB2.UI.CharacterSelectCallback)
		if (characters[i] == API.GetCharacterName()) then
			selectedItem = entry
		end
		control.dropdown:AddItem(entry, false)
	end

	control.dropdown:SelectItem(selectedItem)
end

function RB2.UI.ResearchLabelUpdate(control, key)
	local parent = control:GetParent()
	
--	if (not parent:IsHidden()) then
		local value = parent.dataItem[key]
		if ((value == nil) or (value == "")) then
			RB2.Debug.debug("Key: " .. key)
			control:SetHidden(true)
		else
			control:SetHidden(false)
			if (key == "icon") then
				control:SetTexture(value)
			elseif (key == "name") then
				control:SetText(value)
			elseif (key == "other") then
				control:SetText(value)
			elseif (key == "charname") then
				control:SetText(value)
			elseif (key == "time") then
				local rpi = control:GetParent().dataItem["rpi"]
				value = rpi:getCompletionTime()
				if (value < 0) then				
					RB2.UI.RefreshResearchIcons()
				else
					control:SetText(Utils.formatTime(value))
				end
			end
		end
--	end
end

function RB2.UI.MotifIconUpdate(control, key)
	local parent = control:GetParent()
	
	if (parent.dataItem ~= nil) then
		local value = parent.dataItem[key]
		if ((value == nil) or (value == "")) then
			RB2.Debug.debug("Key: " .. key)
			control:SetHidden(true)
		else
			local known = parent.dataItem.known or false
			local tt = known and "Known" or "Unknown"
			control:SetHidden(false)
			if (key == "icon") then
				control:SetTexture(value)
				if (control.data == nil) then
					control.data = {}
				end
				control.data.tooltipText = tt .. " - Item: " .. parent.dataItem.itemLink
			elseif (key == "text") then
				control:SetText(value)
			end
			if (parent.dataItem.known) then
				control:SetAlpha(1)
			else
				control:SetAlpha(0.25)
			end
			
		end
	end
end

function RB2.UI.PanelResearchInitialized(control)
	RB2.UI.Controls.PanelResearch = control
end

function RB2.UI.PanelKnowledgeInitialized(control)
	RB2.UI.Controls.PanelKnowledge = control
end

function RB2.UI.PanelResearchLineContainerInitialized(control)
	RB2.UI.Controls.PanelResearchLineContainer = control
end

function RB2.UI.PanelKnowledgeLineContainerInitialized(control)
	RB2.UI.Controls.PanelKnowledgeLineContainer = control
end

function RB2.UI.ResearchReminderInitialized(control)
	RB2.UI.Controls.ResearchReminder = control
end

function RB2.UI.KnowledgeReminderInitialized(control)
	RB2.UI.Controls.KnowledgeReminder = control
end

function RB2.UI.PKGridRegisterControl(control, name)
	local dataItem = RB2.UI.PKGetParentDataItem(control)
	dataItem[name] = control
end

function RB2.UI.PKGridRegisterMotifContainer(control)
	RB2.UI.Controls.MotifContainer = control
end

function RB2.UI.PKGetParentDataItem(control)
	local dataItem = nil
	if (control:GetParent().dataItem == nil) then
		dataItem = {}
		control:GetParent().dataItem = dataItem
	else
		dataItem = control:GetParent().dataItem
	end
	return dataItem
end


function RB2.UI.InitUIControls()
	RB2.UI.InitializationPhase = RB2.UI.InitializationPhase + 1
	RB2.UI.InitCharacterSelectControl()
	RB2.UI.PKCreateControls()
	RB2.UI.RefreshPanelKnowledgeControls()
	RB2.UI.PerformPostInitFunctions()	
end

function RB2.UI.PKCreateControls()
	RB2.UI.PKCreateGrid(1, "Weapon traits")
	RB2.UI.PKCreateGrid(2, "Armor traits")
end

function RB2.UI.PKCreateMotifGrid()
	local grid = WINDOW_MANAGER:CreateControlFromVirtual(RB2.UI.Controls.PanelKnowledgeLineContainer:GetName() .. "MotifGrid", 
		RB2.UI.Controls.PanelKnowledgeLineContainer, "RB2MotifGrid")
	grid:SetAnchor(TOPLEFT, RB2.Knowledge.Grids[1], TOPRIGHT, 0, 0)	
	RB2.UI.Controls.MotifGrid = grid
		
	local colIndex = 1
	local controlIndex = 1
	local topOffset = 10
	local leftOffset = 0
	
	for styleItemIndex, mrd in pairs(RB2.MRDC:getMotifRefDataTable()) do
		local container = WINDOW_MANAGER:CreateControlFromVirtual(RB2.UI.Controls.MotifContainer:GetName() .. "Motif" .. controlIndex, 
			RB2.UI.Controls.MotifContainer, "RB2MotifIcon")
		container:SetAnchor(TOPLEFT, RB2.UI.Controls.MotifContainer, TOPLEFT, leftOffset, topOffset)
		leftOffset = leftOffset + RB2.Knowledge.MotifWidth + RB2.Knowledge.MotifXPadding
		
		local dataItem = { icon = mrd.Icon, text = Utils.prettifyItemName(mrd.StyleName), 
			known = false, itemLink = Utils.prettifyItemName(mrd.StyleItemLink) }
		container.dataItem = dataItem		
		RB2.UI.Controls.MotifData[styleItemIndex] = dataItem
						
		colIndex = colIndex + 1
		if (colIndex > RB2.Knowledge.MotifsPerRow) then
			colIndex = 1
			topOffset = topOffset + RB2.Knowledge.MotifHeight + RB2.Knowledge.MotifYPadding
			leftOffset = 0
		end
		
		controlIndex = controlIndex + 1
	end
	
	topOffset = topOffset + RB2.Knowledge.MotifHeight + RB2.Knowledge.MotifYPadding
	local label = WINDOW_MANAGER:CreateControl(RB2.UI.Controls.MotifContainer:GetName() .. "DwemerLabel", 
		RB2.UI.Controls.MotifContainer, CT_LABEL)
	label:SetAnchor(TOP, RB2.UI.Controls.MotifContainer, TOP, 0, topOffset)
	label:SetFont("ZoFontGameSmall")
	label:SetColor(255,255,255)
	label:SetText("Dwemer chapters")
	
	RB2.UI.Controls.MotifDwemerIcons = { }
	
	colIndex = 1
	leftOffset = 22
	topOffset = topOffset + 20
	local indexes = MoKnC.getChapterIndexes()
	for i = 1, #indexes do
		local idx = indexes[i]
		local backdrop = WINDOW_MANAGER:CreateControl(RB2.UI.Controls.MotifContainer:GetName() .. "DwemerIconBackdrop" .. i, 
			RB2.UI.Controls.MotifContainer, CT_TEXTURE)
		backdrop:SetTexture("/esoui/art/crafting/crafting_alchemy_badslot.dds")
		backdrop:SetDimensions(24, 24)
		backdrop:SetAlpha(0.5)
		backdrop:SetAnchor(TOPLEFT, RB2.UI.Controls.MotifContainer, TOPLEFT, leftOffset, topOffset)
		local vIcon = WINDOW_MANAGER:CreateControlFromVirtual(RB2.UI.Controls.MotifContainer:GetName() .. "DwemerIcon" .. i,
			 backdrop, "ZO_Options_WarningIcon")
		vIcon:SetAnchor(TOPLEFT, RB2.UI.Controls.MotifContainer, TOPLEFT, leftOffset, topOffset)
		vIcon:SetTexture(MoKnC:getChapterIcon(idx))
		
		local dataItem = { chapterIndex = idx, name = MoKnC:getChapterName(idx), known = false, backdrop = backdrop }
		vIcon.dataItem = dataItem
		
		vIcon:SetDimensions(24, 24)
		vIcon:SetInheritAlpha(false)
		vIcon:SetAlpha(1)
		vIcon.data = { tooltipText = dataItem.name }
		leftOffset = leftOffset + 30
		
		colIndex = colIndex + 1
		
		table.insert(RB2.UI.Controls.MotifDwemerIcons, vIcon)
		
		if (colIndex == 8) then
			colIndex = 1
			leftOffset = 22
			topOffset = topOffset + 30
		end
	end
end

function RB2.UI.PKCreateGrid(index, title)
	local grid = WINDOW_MANAGER:CreateControlFromVirtual(RB2.UI.Controls.PanelKnowledgeLineContainer:GetName() .. "Grid" .. index, 
		RB2.UI.Controls.PanelKnowledgeLineContainer, "RB2KnowledgeGrid")
	local top = (index - 1) * grid:GetHeight() + (index - 1) * (RB2.Knowledge.GridSpacing * 2)
	grid:SetAnchor(TOPLEFT, RB2.UI.Controls.PanelKnowledgeLineContainer, TOPLEFT, 10, top)
	grid.dataItem.title = title
	RB2.Knowledge.Grids[index] = grid

	RB2.UI.PKCreateGridColumns(index, grid)
	RB2.UI.PKCreateGridRows(index, grid)
	RB2.UI.PKCreateGridCells(index, grid)
	
	if (index == 1) then
		RB2.UI.PKCreateMotifGrid()
	end
end

function RB2.UI.PKCreateGridColumns(index, grid)
	grid.dataItem.columnControls = {}
	local previousControl = grid
	local previousAnchorPoint = TOPLEFT
	local anchorXOffset = 10
	local anchorYOffset = 40
	local iconColumnCount = 0
	local rrdc = RB2.RRDC

	local columnTotalHeight = RB2.Knowledge.RowHeight * (#RB2.Knowledge.KnowledgeTables[index] + 2)
	for i = 1, #RB2.Knowledge.ColumnWidths[index] do
		local columnHeader = RB2.Knowledge.ColumnHeaders[index][i]
		local columnType = RB2.Knowledge.ColumnTypes[index][i]
		local columnWidth = RB2.Knowledge.ColumnWidths[index][i]
		local resKey = RB2.Knowledge.KnowledgeTables[index][1][i]

		local column = WINDOW_MANAGER:CreateControl(grid:GetName() .. "Column" .. i, grid, CT_CONTROL)
		column:SetDimensions(columnWidth, columnTotalHeight)
		column:SetAnchor(TOPLEFT, previousControl, previousAnchorPoint, anchorXOffset, anchorYOffset)

		local backdrop = WINDOW_MANAGER:CreateControl(column:GetName() .. "Background", column, CT_TEXTURE)
		--		backdrop:SetTexture("/EsoUI/Art/Miscellaneous/listItem_highlight.dds")

		if (columnType == "I") then -- icon
			backdrop:SetAnchor(TOPLEFT, column, TOPLEFT, 0, RB2.Knowledge.RowHeight)
			backdrop:SetAnchor(BOTTOMRIGHT, column, BOTTOMRIGHT, 0, 0)
			if (iconColumnCount % 2 == 0) then
				backdrop:SetColor(0,128,255)
				backdrop:SetAlpha(0.075)
			else
				backdrop:SetColor(255,128,0)
				backdrop:SetAlpha(0)
			end
			iconColumnCount = iconColumnCount + 1
			
			local sumLabel = WINDOW_MANAGER:CreateControl(column:GetName() .. "Label" .. i, grid, CT_LABEL)
			sumLabel:SetAnchor(TOP, column, BOTTOM, 0, 2)
			sumLabel:SetFont("ZoFontGameSmall")
			sumLabel:SetText("0") 
			sumLabel:SetColor(255,255,255)
			sumLabel:SetAlpha(0.5)
			local rrd = rrdc:getResearchRefDataByKey(resKey)
			sumLabel.dataItem = { tradeSkillType = rrd.tradeSkillType, researchLineIndex = rrd.researchLineIndex }
			
			sumLabel.refresh = function(self)
				local rpc = RB2.CDM:getRPCOfSelectedCharacter()
				self:SetText(rpc:countKnownTraitsForResKey(self.dataItem.tradeSkillType, self.dataItem.researchLineIndex))								
			end
			
			table.insert(RB2.UI.Controls.KnowledgeSummaries, sumLabel)
		elseif (columnType == "D") then
			local xOffset = (RB2.Knowledge.DividerColumnWidth - RB2.Knowledge.DividerWidth) / 2
			backdrop:SetDimensions(RB2.Knowledge.DividerWidth, columnTotalHeight)
			backdrop:SetAnchor(TOPLEFT, column, TOPLEFT, xOffset, 0)
			backdrop:SetAnchor(BOTTOMRIGHT, column, BOTTOMRIGHT, -1 * xOffset, 0)
			backdrop:SetColor(255,255,255)
			backdrop:SetAlpha(0.125)
			iconColumnCount = 0
		else
			backdrop:SetAnchor(TOPLEFT, column, TOPLEFT, 0, 0)
			backdrop:SetAnchor(BOTTOMRIGHT, column, BOTTOMRIGHT, 0, 0)
			backdrop:SetAlpha(0)
			iconColumnCount = 0
		end

		if (columnHeader ~= nil) then
			local ch = WINDOW_MANAGER:CreateControl(column:GetName() .. "Header", column, CT_LABEL)
			ch:SetAnchor(CENTER, column, TOP, 0, RB2.Knowledge.RowHeight / 2)
			ch:SetFont("ZoFontGame")
			ch:SetColor(255,255,255)
			ch:SetText(columnHeader)
			ch:SetResizeToFitDescendents(true)
		end

		if (resKey ~= "") then
			local rrd = rrdc:getResearchRefDataByKey(resKey)
			local vIcon = WINDOW_MANAGER:CreateControlFromVirtual(column:GetName() .. "Icon", column, "ZO_Options_WarningIcon")
			vIcon:SetAnchor(CENTER, backdrop, TOP, 0, RB2.Knowledge.RowHeight / 2)
			vIcon:SetTexture(rrd.icon)
			vIcon:SetDimensions(RB2.Knowledge.IconWidth, RB2.Knowledge.IconHeight)
			vIcon.data = { tooltipText = rrd.researchLineName }
		end

		table.insert(grid.dataItem.columnControls, column)
		previousControl = column
		previousAnchorPoint = TOPRIGHT
		anchorXOffset = 0
		anchorYOffset = 0
	end
end

function RB2.UI.PKCreateGridRows(index, grid)
	grid.dataItem.rowControls = {}
	local rowCount = #RB2.Knowledge.KnowledgeTables[index]

	local totalRowWidth = 10
	for i = 1, #RB2.Knowledge.ColumnWidths[index] do
		totalRowWidth = totalRowWidth + RB2.Knowledge.ColumnWidths[index][i]
	end

	for i = 1, rowCount do
		local column1 = grid.dataItem.columnControls[1]
		local row = WINDOW_MANAGER:CreateControl(grid:GetName() .. "Row" .. i, grid, CT_CONTROL)
		row:SetDimensions(totalRowWidth, RB2.Knowledge.RowHeight)
		row:SetAnchor(TOPLEFT, grid, TOPLEFT, 0, (column1:GetTop() - grid:GetTop()) + RB2.Knowledge.RowHeight * (i + 1))

		local backdrop = WINDOW_MANAGER:CreateControl(row:GetName() .. "Background", row, CT_TEXTURE)
		backdrop:SetAnchorFill()
		if (i % 2 == 0) then
			backdrop:SetColor(0,128,255)
			backdrop:SetAlpha(0)
		else
			backdrop:SetColor(255,128,0)
			backdrop:SetAlpha(0.075)
		end

		table.insert(grid.dataItem.rowControls, row)
	end
end

function RB2.UI.PKCreateGridCells(index, grid)
	grid.dataItem.cellControls = {}
	local rrdc = RB2.RRDC
	local kt = RB2.Knowledge.KnowledgeTables[index]
	local cts = RB2.Knowledge.ColumnTypes[index]

	for ridx = 1, #kt do
		local rowData = kt[ridx]
		for cidx = 1, #rowData do
			local columnType = cts[cidx]
			local resKey = rowData[cidx]
			local column = grid.dataItem.columnControls[cidx]

			if (columnType == "C") then
				resKey = rowData[3]
				local rrd = rrdc:getResearchRefDataByKey(resKey)
				local ch = WINDOW_MANAGER:CreateControl(column:GetName() .. "Row" .. ridx .. "Caption", column, CT_LABEL)
				ch:SetAnchor(RIGHT, column, TOPRIGHT, 0, RB2.Knowledge.RowHeight * (ridx + 1.5))
				ch:SetFont("ZoFontGame")
				ch:SetColor(255,255,255)
				ch:SetText(rrd.researchLineTraitDescription)
				ch:SetResizeToFitDescendents(true)
			elseif (columnType == "I") then
				local rrd = rrdc:getResearchRefDataByKey(resKey)
				local cellIcon = WINDOW_MANAGER:CreateControlFromVirtual(column:GetName() .. "Icon" .. ridx, column, "ZO_Options_WarningIcon")
				cellIcon:SetAnchor(TOP, column, TOP, 0, RB2.Knowledge.RowHeight * (ridx + 1) + 3)
				cellIcon:SetDimensions(RB2.Knowledge.IconWidth, RB2.Knowledge.IconHeight)
				cellIcon.dataItem = { resKey = resKey }
				cellIcon.refreshIcon = function(self)
					local rpc = RB2.CDM:getRPCOfSelectedCharacter()
					local rrd = RB2.RRDC:getResearchRefDataByKey(self.dataItem.resKey)
					local rpi = rpc:getByResearchKey(self.dataItem.resKey)
					self.data = { tooltipText = rrd.researchLineTraitDescription }
					-- Adding remaining time to tooltip. Since I found no way to refresh the tooltip text
					-- (when it is shown on the UI) every second, the tooltip doesn't show the seconds value in the remaining time display.
					if (rpi.status == rpi.StatusType.INPROGRESS) then
						self.data =  { tooltipText = rrd.researchLineTraitDescription .. " (" .. Utils.formatTime(rpi:getCompletionTime(), true) .. ")" }
					end

					local charsKnowingRes, charsNotKnowingRes = RB2.CDM:getNonSelectedCharactersWithResearch(self.dataItem.resKey, true)

					if (rpi.status == rpi.StatusType.INPROGRESS) then
						-- Show the viewer the trait is currently being researched.
						self:SetTexture("/esoui/art/mappins/group_pin.dds")
						self:SetAlpha(1)
					elseif ((rpi.status == rpi.StatusType.KNOWN) or (rpi.status == rpi.StatusType.COMPLETED)) then
						-- Show the viewer the trait is learned.
						self:SetTexture("/esoui/art/mappins/follower_pin.dds")
						if ((#charsNotKnowingRes > 0) and (RB2.SettingsHandler.getShowOthercharactersOnKnowledgePanel())) then
							self:SetAlpha(0.25)
							self.data.tooltipText = self.data.tooltipText .. " (Unknown for: " .. table.concat(charsNotKnowingRes, ", ") .. ")"
						else
							self:SetAlpha(0.75)
						end
					else
						local texture = "/esoui/art/mappins/hostile_pin.dds"
						local inventoryContainer = RB2.CDM:getICOfSelectedCharacter()
						local bankContainer = RB2.BankContainer
						
						if (inventoryContainer:hasItemForResearchKey(self.dataItem.resKey) or 
							bankContainer:hasItemForResearchKey(self.dataItem.resKey)) then
							self:SetAlpha(.75)
						else
							self:SetAlpha(.25)
							if ((#charsKnowingRes > 0) and (RB2.SettingsHandler.getShowOthercharactersOnKnowledgePanel())) then
								texture = "/esoui/art/tooltips/arrow_down.dds"
								--texture = "/esoui/art/ava/avacapturebar_fill_contested_ebonheart.dds"
								self.data.tooltipText = self.data.tooltipText .. " (Known by: " .. table.concat(charsKnowingRes, ", ") .. ")" 
							end
						end
						self:SetTexture(texture)
					end
				end

				table.insert(grid.dataItem.cellControls, cellIcon)
			end
		end
	end
end

function RB2.UI.HideControllerWithUI()
	if (not RB2.UI.Controls.MainPanel:IsHidden()) then
		RB2.UI.ShowHideController(false, true)
	end
end

function RB2.UI.ShowControllerWithUI()
	if (RB2.SettingsObject:getControllerShown()) then
		RB2.UI.ShowHideController(true, true)
	end
end

function RB2.UI.ShowHideController(value, disableSettingsWriteBack)
	RB2.Debug.debug("ShowHideController: " .. (value and "true" or "false"))
	if (RB2.UI.Controls.MainPanel ~= nil) then
		RB2.UI.Controls.MainPanel:SetHidden(not value)
		RB2.UI.Controls.MainPanel:SetAlpha(value and 1 or 0)
		if (not disableSettingsWriteBack) then
			RB2.SettingsObject:setControllerShown(value)
		end
		if (not value) then
			RB2.UI.HidePanelResearch()
			RB2.UI.HidePanelKnowledge()
		end
	end
end

function RB2.UI.HideController()
	RB2.UI.ShowHideController(false)
end

function RB2.UI.ToggleController()
	if (RB2.UI.ControllerTempShown) then
		RB2.UI.ControllerTempShown = false
		RB2.UI.ShowHideController(true)
	else
		RB2.UI.ShowHideController(RB2.UI.Controls.MainPanel:IsHidden())
	end
end

function RB2.UI.ShowControllerTemp()
	if (RB2.UI.Controls.MainPanel:IsHidden()) then
		RB2.Debug.debug("Temporarily showing control panel")
		RB2.UI.ControllerTempShown = true
		RB2.UI.ShowHideController(true)
		local timeout = RB2.SettingsObject:getControllerFadeTime() * 1000
		zo_callLater(RB2.UI.FadeController, timeout)
	end
end

function RB2.UI.FadeController()
	if(RB2.UI.ControllerTempShown) then
		local anim, timeline = CreateSimpleAnimation(ANIMATION_ALPHA, RB2.UI.Controls.MainPanel)
		anim:SetAlphaValues(1, 0)
		anim:SetDuration(500)
		anim:SetHandler("OnStop", function()
			RB2.Debug.debug("Done playing")
			RB2.UI.ControllerTempShown = true
			RB2.UI.ShowHideController(false)
		end)
		RB2.Debug.debug("Start playing")
		timeline:SetPlaybackType(ANIMATION_PLAYBACK_ONE_SHOT, 1)
		timeline:PlayFromStart()
	end
end


----------------------------------------------------------------------------------------
-- callbacks

function RB2.UI.PerformPostInitFunctions()
	if (RB2.UI.InitializationPhase < 2) then
		zo_callLater(RB2.UI.PerformPostInitFunctions, 100)
	else
		RB2.Debug.debug("Post init")
		if ((RB2.SettingsObject:getLeft() ~= -1) and (RB2.SettingsObject:getTop() ~= -1)) then
			RB2.UI.Controls.MainPanel:ClearAnchors()
			RB2.UI.Controls.MainPanel:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, RB2.SettingsObject:getLeft(), RB2.SettingsObject:getTop())
		end
		
		local ra = RB2.SettingsObject:getPanelResearchAnchor()
		if (ra ~= nil) then
			RB2.UI.Controls.PanelResearch:ClearAnchors()
			RB2.UI.Controls.PanelResearch:SetAnchor(ra[1], GuiRoot, ra[2], ra[3], ra[4])
		end

		local ka = RB2.SettingsObject:getPanelKnowledgeAnchor()
		if (ka ~= nil) then
			RB2.UI.Controls.PanelKnowledge:ClearAnchors()
			RB2.UI.Controls.PanelKnowledge:SetAnchor(ka[1], GuiRoot, ka[2], ka[3], ka[4])
		end
		
		RB2.UI.Controls.MainPanel:SetMovable(not RB2.SettingsObject:getLocked())
		RB2.UI.Controls.PanelResearch:SetMovable(not RB2.SettingsObject:getLocked())
		RB2.UI.Controls.PanelKnowledge:SetMovable(not RB2.SettingsObject:getLocked())
		RB2.UI.ShowHideController(RB2.SettingsObject:getControllerShown())
	end
end

function RB2.UI.CharacterSelectCallback(control, characterName)
	RB2.CDM:selectCharacter(characterName)
	RB2.UI.RefreshResearchIcons()
	RB2.UI.RefreshPanelKnowledgeControls()
end

function RB2.UI.OnMainPanelMouseUp(control)
	RB2.SettingsObject:storeMainPanelPositions(RB2.UI.Controls.MainPanel:GetTop(), RB2.UI.Controls.MainPanel:GetLeft())
end

function RB2.UI.OnResearchPanelMouseUp(control)
	if (not RB2.SettingsObject:getLocked()) then
		local valid,anchorPoint,relativeTo,relativePoint,offsetX,offsetY = RB2.UI.Controls.PanelResearch:GetAnchor()
--		RB2.Debug.error(relativeTo:GetName() .. " -> Top: " .. offsetY .. ", Left: " .. offsetX .. " (" .. anchorPoint .. ", " .. relativePoint .. ")")
		RB2.SettingsObject:storeResearchPanelAnchor({ anchorPoint, relativePoint, offsetX, offsetY })
	else
--		RB2.Debug.error("Not movable, not setting")
	end
end

function RB2.UI.OnKnowledgePanelMouseUp(control)
	if (not RB2.SettingsObject:getLocked()) then
		local valid,anchorPoint,relativeTo,relativePoint,offsetX,offsetY = RB2.UI.Controls.PanelKnowledge:GetAnchor()
--		RB2.Debug.error(relativeTo:GetName() .. " -> Top: " .. offsetY .. ", Left: " .. offsetX .. " (" .. anchorPoint .. ", " .. relativePoint .. ")")
		RB2.SettingsObject:storeKnowledgePanelAnchor({ anchorPoint, relativePoint, offsetX, offsetY })
	else
--		RB2.Debug.error("Not movable, not setting")
	end
end

function RB2.UI.OnButtonResearchClick(control)
	if (RB2.UI.PanelResearchOpened) then
		RB2.UI.HidePanelResearch()
	else
		RB2.UI.ShowPanelResearch()
	end
end

function RB2.UI.OnButtonKnowledgeClick(control)
	if (RB2.UI.PanelKnowledgeOpened) then
		RB2.UI.HidePanelKnowledge()
	else
		RB2.UI.ShowPanelKnowledge()
	end
end

function RB2.UI.TogglePanelResearch()
	if (RB2.UI.PanelResearchOpened) then
		RB2.UI.HidePanelResearch()
	else
		RB2.UI.ShowPanelResearch()
	end
end

function RB2.UI.ShowPanelResearch()
	RB2.UI.HidePanelKnowledge()

	if (not RB2.SettingsObject:getReminderWasShown()) then
		RB2.UI.Controls.ResearchReminder:SetAlpha(1)
		RB2.SettingsObject:setReminderWasShown()
	else
		RB2.UI.Controls.ResearchReminder:SetAlpha(0)
	end
	
	RB2.UI.PanelResearchOpened = true
	RB2.UI.RefreshPanelResearchData()
	RB2.UI.BuildPanelResearch()
	RB2.UI.Controls.PanelResearch:SetHidden(false)
	RB2.UI.Controls.ButtonResearch:SetAlpha(0.3)
	if(RB2.UI.Controls.MainPanel:IsHidden()) then
		RB2.UI.ShowControllerTemp()
	end
end

function RB2.UI.HidePanelResearch()
	RB2.UI.PanelResearchOpened = false
	RB2.UI.Controls.PanelResearch:SetHidden(true)
	RB2.UI.Controls.ButtonResearch:SetAlpha(1)
end

function RB2.UI.TogglePanelKnowledge()
	if (RB2.UI.PanelKnowledgeOpened) then
		RB2.UI.HidePanelKnowledge()
	else
		RB2.UI.ShowPanelKnowledge()
	end
end

function RB2.UI.ShowPanelKnowledge()
	RB2.UI.HidePanelResearch()

	if (not RB2.SettingsObject:getReminderWasShown()) then
		RB2.UI.Controls.KnowledgeReminder:SetAlpha(1)
		RB2.SettingsObject:setReminderWasShown()
	else
		RB2.UI.Controls.KnowledgeReminder:SetAlpha(0)
	end

	RB2.UI.PanelKnowledgeOpened = true
	RB2.UI.RefreshPanelKnowledgeData()
	RB2.UI.Controls.PanelKnowledge:SetHidden(false)
	RB2.UI.Controls.ButtonKnowledge:SetAlpha(0.3)
	RB2.UI.PKStartRefreshTimer()
	if(RB2.UI.Controls.MainPanel:IsHidden()) then
		RB2.UI.ShowControllerTemp()
	end
end

function RB2.UI.HidePanelKnowledge()
	RB2.UI.PanelKnowledgeOpened = false
	RB2.UI.Controls.PanelKnowledge:SetHidden(true)
	RB2.UI.Controls.ButtonKnowledge:SetAlpha(1)
end


-- refresh functions

function RB2.UI.RefreshPanelResearchData()
	RB2.Debug.debug("refresh panel research")
	RB2.CDM:refreshRPCs(RB2.SettingsHandler.getShowOthercharactersOnResearchPanel(), true)
--	if (RB2.SettingsHandler.getShowOthercharactersOnResearchPanel()) then
--		RB2.CDM:refreshInProgressRPCOfSelectedAndComparisonCharacters()
--	else
--		local rpc = RB2.CDM:getRPCOfSelectedCharacter()
--		rpc:refreshInProgress()
--	end
	if (RB2.UI.PanelResearchOpened) then
		zo_callLater(RB2.UI.RefreshPanelResearchData, 1000)
	end
end

function RB2.UI.RefreshPanelKnowledgeData()
	RB2.Debug.debug("refresh panel knowledge")
	RB2.CDM:refreshRPCs(RB2.SettingsHandler.getShowOthercharactersOnResearchPanel(), false)
--	if (RB2.SettingsHandler.getShowOthercharactersOnResearchPanel()) then
--		RB2.CDM:refreshRPCOfSelectedAndComparisonCharacters()
--	else
--		local rpc = RB2.CDM:getRPCOfSelectedCharacter()
--		rpc:refreshAll()
--	end
	if (RB2.UI.PanelKnowledgeOpened) then
		zo_callLater(RB2.UI.RefreshPanelKnowledgeData, 1000)
	end
end

function RB2.UI.RefreshPanelKnowledgeControls()
	if (#RB2.Knowledge.Grids > 0) then
		RB2.UI.PKGridBeforeShow(RB2.Knowledge.Grids[1])
		RB2.UI.PKGridBeforeShow(RB2.Knowledge.Grids[2])
		RB2.UI.PKGridSummariesBeforeShow()
		RB2.UI.PKMotifGridBeforeShow()
	end
end

function RB2.UI.PKGridBeforeShow(control)
	control.dataItem.titleControl:SetText(control.dataItem.title)
	for i = 1, #control.dataItem.cellControls do
		local cell = control.dataItem.cellControls[i]
		cell:refreshIcon(cell)
	end
end

function RB2.UI.PKGridSummariesBeforeShow()
	for i = 1, #RB2.UI.Controls.KnowledgeSummaries do
		local control = RB2.UI.Controls.KnowledgeSummaries[i]
		control:refresh()
	end
end

function RB2.UI.PKMotifGridBeforeShow()
	local mkc = RB2.CDM:getMKCOfSelectedCharacter()
	for styleItemIndex, known in pairs(mkc:getMotifs()) do
		local dataItem = RB2.UI.Controls.MotifData[styleItemIndex]
		if (dataItem ~= nil) then
			dataItem.known = known
		else
			RB2.Debug.error("Motif dataItem not found!")
		end
	end
	for i = 1, #RB2.UI.Controls.MotifDwemerIcons do
		local icon = RB2.UI.Controls.MotifDwemerIcons[i]
		icon.dataItem.known = mkc:isChapterKnown(icon.dataItem.chapterIndex)
		icon.data.tooltipText = icon.dataItem.name .. " - " .. (icon.dataItem.known and "Known" or "Unknown")
		if (icon.dataItem.known) then
			icon.dataItem.backdrop:SetAlpha(0)			
			icon:SetAlpha(1)
		else
			icon.dataItem.backdrop:SetAlpha(0.5)
			icon:SetAlpha(0.5)
		end
	end
end

function RB2.UI.PKStartRefreshTimer()
	if (not RB2.UI.PanelKnowledgeRefreshStarted) then
		-- 10 second refresh should be ok on the Knowledge Panel since it doesn't show seconds	
		zo_callLater(RB2.UI.PKPerformRefreshOfActiveResearches, 10000)
		RB2.UI.PanelKnowledgeRefreshStarted = true
	end
end

function RB2.UI.PKPerformRefreshOfActiveResearches()
	RB2.UI.PanelKnowledgeRefreshStarted = false
	RB2.Debug.debug("Refreshing research")
	local rpc = RB2.CDM:getRPCOfSelectedCharacter()
	rpc:refreshInProgress()
	RB2.UI.RefreshPanelKnowledgeControls()
	if (RB2.UI.PanelKnowledgeOpened) then
		RB2.Debug.debug("Scheduling next refresh.")
		RB2.UI.PKStartRefreshTimer()
	end
end


function RB2.UI.RefreshResearchIcon(control, max, current, idx)
	local tracker = RB2.SettingsHandler.getTracker(idx)

	if (tracker) then
		if (current == 0) then
			control:SetColor(255,0,0)
		elseif(current < max) then
			control:SetColor(255,255,0)
		elseif(current == max) then
			control:SetColor(0,255,0)
		else
			control:SetColor(0,255,255)
		end
		control:SetAlpha(1)
	else
		control:SetColor(255,255,255)
		control:SetAlpha(0.5)
	end
end

function RB2.UI.RefreshResearchIcons()
	if (RB2.SettingsHandler.getShowOthercharactersOnResearchPanel()) then
		RB2.CDM:refreshRPCOfSelectedAndComparisonCharacters()
	else
		local rpc = RB2.CDM:getRPCOfSelectedCharacter()
		rpc:refreshAll()
	end

	local researches, charResNums, allMaxResearches, currResearches = RB2.CDM:getInProgressResearchData(RB2.SettingsHandler.getShowOthercharactersOnResearchPanel())

--	RB2.UI.ScheduleResearchRefresh()

	RB2.UI.RefreshResearchIcon(RB2.UI.Controls.IconSmithing, 
		allMaxResearches[Config.TradeSkillTypes[1].Type], currResearches[Config.TradeSkillTypes[1].Type], 1)
	RB2.UI.RefreshResearchIcon(RB2.UI.Controls.IconClothing, 
		allMaxResearches[Config.TradeSkillTypes[2].Type], currResearches[Config.TradeSkillTypes[2].Type], 2)
	RB2.UI.RefreshResearchIcon(RB2.UI.Controls.IconWoodworking, 
		allMaxResearches[Config.TradeSkillTypes[3].Type], currResearches[Config.TradeSkillTypes[3].Type], 3)

	RB2.UI.RefreshPanelResearchData()	
	RB2.UI.BuildPanelResearch()
end

function RB2.UI.BuildPanelResearchHeaderRow(control, tstName, maxResearches, activeResearches)
	control:SetHidden(false)
	local dataItem = { icon = "", name = "", other = "", charname = "", time = "", rpi = nil }
	if (maxResearches < activeResearches) then
		dataItem.name = RB2.Colors.Teal
	elseif (maxResearches == activeResearches) then
		dataItem.name = RB2.Colors.BrightGreen
	elseif (activeResearches == 0) then
		dataItem.name = RB2.Colors.BrightRed
	else
		dataItem.name = RB2.Colors.Yellow
	end 
	dataItem.name = dataItem.name .. tstName .. " (" .. activeResearches .. "/" .. maxResearches .. ")"				
	control.dataItem = dataItem
	control:GetChild(1):SetHidden(true)
	control:GetChild(2):SetHidden(false)
	control:GetChild(3):SetHidden(true)
	control:GetChild(4):SetHidden(true)
	control:GetChild(5):SetHidden(true)
end

function RB2.UI.BuildPanelResearchItemRow(control, rpi, maxResearches, activeResearches)
	control:SetHidden(false)
	local dataItem = { icon = "", name = "", other = "", charname = "", time = "", rpi = rpi }
	
	local rrd = RB2.RRDC:getResearchRefDataByKey(rpi:getKey())
	dataItem.icon = rrd.icon
	
	dataItem.name = "   " .. rrd.researchLineName
	dataItem.other = rrd.researchLineTraitDescription
	dataItem.time = "x"
	
	if (maxResearches < activeResearches) then
		dataItem.charname = RB2.Colors.Teal
	elseif (maxResearches <= activeResearches) then
		dataItem.charname = RB2.Colors.BrightGreen
	elseif (activeResearches == 0) then
		dataItem.charname = RB2.Colors.BrightRed
	else
		dataItem.charname = RB2.Colors.Yellow
	end 
	dataItem.charname = dataItem.charname .. rpi.characterName
	
	control.dataItem = dataItem
	control:GetChild(1):SetHidden(false)
	control:GetChild(2):SetHidden(false)
	control:GetChild(3):SetHidden(false)
	control:GetChild(4):SetHidden(false)
	control:GetChild(5):SetHidden(false)
end

function RB2.UI.BuildPanelResearch()
	local container = RB2.UI.Controls.PanelResearchLineContainer
	
	local researches, charResNums, allMaxResearches = RB2.CDM:getInProgressResearchData(RB2.SettingsHandler.getShowOthercharactersOnResearchPanel())
	
	researches = RB2.CDM:getRPCOfSelectedCharacter():sortInProgressResearches(researches)
	
	local needed = #researches + 3
	local numChildren = container:GetNumChildren()

	RB2.Debug.debug("Numchildren: " .. numChildren .. ", needed: " .. needed)

	if (numChildren > needed) then
		for i = numChildren, needed + 1, -1 do
			local child = container:GetChild(i)
			if (child ~= nil) then
				child:SetHidden(true)
			end
		end
	elseif (numChildren < needed) then
		for i = numChildren + 1, needed do
			local anchorControl = nil
			local anchorRelativePoint = nil
			
			if (i == 1) then
				anchorControl = container
				anchorRelativePoint = TOPLEFT
			else
				anchorControl = container:GetChild(i - 1)
				anchorRelativePoint = BOTTOMLEFT
			end

			local name = container:GetName() .. "Row" .. i
			local control = WINDOW_MANAGER:CreateControlFromVirtual(name, container, "RB2ResearchLine")
			control:SetAnchor(TOPLEFT, anchorControl, anchorRelativePoint, 0, 2)
			control:SetParent(container)
			RB2.Debug.debug("Created: " .. name .. ", anchor: " .. anchorControl:GetName())
		end
	end
	
	local controlId = 1
	local currentTst = 0
	
	for i = 1, #Config.TradeSkillTypes do
		local tst = Config.TradeSkillTypes[i].Type
		--local rpis = rpc:listInProgressResearches(tst)
		local rpis = {}
		for i = 1, #researches do
			local rpi = researches[i]
			if (rpi.tradeSkillType == tst) then
				table.insert(rpis, rpi)
			end
		end
		local maxResearches =  allMaxResearches[tst]
		local activeResearches = #rpis
		
		local control = nil 
		if (currentTst ~= tst) then
			currentTst = tst
			control = container:GetChild(controlId)
							
			RB2.Debug.debug("Building header for row: " .. controlId)
			RB2.UI.BuildPanelResearchHeaderRow(control, Config.TradeSkillTypes[i].Name, maxResearches, activeResearches)
			
			controlId = controlId + 1
		end

		for j = 1, activeResearches do
			control = container:GetChild(controlId)
			
			local rpi = rpis[j]
			
			RB2.Debug.debug("Building item for row: " .. controlId)			
			RB2.UI.BuildPanelResearchItemRow(control, rpi, charResNums[rpi.characterName][currentTst][1], charResNums[rpi.characterName][currentTst][2])
			controlId = controlId + 1
		end
	end	
end
